SystemOrganization addCategory: #'SQueakLite-Base'!

Object subclass: #SQueakLConnection
	instanceVariableNames: 'connection connectionName accelerated'
	classVariableNames: 'DataTypeMap SQueakLConnectionTypes'
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLConnection commentStamp: 'MartinLeo 8/2/2010 11:19' prior: 0!
A SQueakLConnection is the parent Strategy of a strategy pattern,
where different types of concrete access strategies can be implemented to 
interact easily with different databases.

("Easily" because SqueakDBX requires somewhat more indepth DB knowlege)

SQueakLConnection class>>create: is the starting point of all operations.
This method is also intended to use the factory method paradigm. 
It can generate multiple connection objects so that the UI can work with multiple connections simultaneously.

For example:

connection1 := SQueakLConnection create: 'ORACLE'.
connection2 := SQueakLConnection create: 'ORACLE'.

connection3 := SQueakLConnection create: 'MYSQL'.
connection4 := SQueakLConnection create: 'POSTGRES'.
connection5 := SQueakLConnection create: 'POSTGRES'.

The connections are initialized with the relevant connection information to different hosts and dastabases, even ports and then they can be stored in a collection and multiple connections can hence be manipulated simulaneously.

Instance Variables
	connection:		<Object>
	connectionName:		<Object>
	accelerated 
	
connection
	- This holds an instance of our DB driver, DB commands are executed through it.

connectionName
	- The name of this particular connection / instance, because we can support more than one connection.

accelerated
	- This boolean stores whether connection pool is active or not. if set to true,
	SQueakLite will aggressively reuse open connections, if set to false,
	SQueakLite will diligently open and close connections (safe mode).
	
The following methods need to be implemented in its concrete subclasses / strategies (please see any concrete stratgegy implemention for more detail).
Expect to see duplicated code. This is by intent. Methods for access databases vary widely.
Often it is a conincidence when they look the same, but future customizations will definately change the syntax. 
It was by design that these data dictionary manipulating methods be implemented seperately from the start, even if the illusion of code duplication exists.

alterTable:column: (takes a column action decorator object and performs the requested action on the table)
alterTable:index:  (takes an index action decorator object and performs the requested action on the table)
connect:port:database:userName:userPassword: (takes the necessary details required to create a DB connection)
createSchema: (takes a schema object and tries to create it)
createTable: (takes a table object and tries to create it)
discoverColumns: (these are helper methods that satisfy the template method populateTable: )
discoverIndex: (these are helper methods that satisfy the template method populateTable: )
dropSchema: (takes a schema object and tries to drop it)
dropTable: (takes a table object and tries to drop it)
generateCreateIndexStmts: (takes a table object and generates the necessary DDLS to create its indexes)
generateCreateTableStmts: (takes a table object and generates the necessary DDLS to create it)
indexes: (takes a schema object and look for all indexes inside this schema)
populateIndexDetail: (takes a index object and populates its details)
schemas (show's all schemas the current connection has access to)
tables: (takes a schema object and looks for all tables inside this schema)
views: (takes a schema object and looks for all views inside this schema)


Future work:
Anticipated future work would include:
1. Native database type support (including modifications or workarounds to OpenDBX to cope with the more exotic data types)
2. Improve Index Support (building different types of indexes)
3. Improved user / schema support
4. Tuning / DBA Views / Management support
5. Editable Data Grid
!

SQueakLConnection subclass: #SQueakLConnMYSQL
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLConnMYSQL commentStamp: 'MartinLeo 7/31/2010 00:16' prior: 0!
A SQueakLConnMYSQL is a concrete Strategy for connecting to a MYSQL Database.

Instance Variables
!

!SQueakLConnMYSQL class methodsFor: 'information methods' stamp: 'MartinLeo 7/10/2010 20:56'!
connectionDatabase
	^'MYSQL'! !

!SQueakLConnMYSQL class methodsFor: 'information methods' stamp: 'MartinLeo 7/10/2010 21:59'!
defaultPort
	^'3306'! !

!SQueakLConnMYSQL methodsFor: 'table methods' stamp: 'MartinLeo 7/30/2010 23:17'!
alterTable: aSQueakLTable column: aSQueakLTableColumnAction
	
	"allows you to modify aSQueakLTable, the action is encapsulated within SQueakLTableColumnAction>>action.
		It is a subclass of aSQueakLTableColumn
		options are: ADD RENAME MODIFY(change column type /length) DROP"
	
	|alterTableCommand prevTblColDef colType ddlError|

	alterTableCommand := 'ALTER TABLE ', aSQueakLTable schemaName , '.', aSQueakLTable table , ' '.

	(aSQueakLTableColumnAction action = 'RENAME')
	ifTrue: [	alterTableCommand := alterTableCommand , ' CHANGE ', aSQueakLTableColumnAction field , ' ', aSQueakLTableColumnAction newName]
	ifFalse: [ alterTableCommand := alterTableCommand , aSQueakLTableColumnAction action, ' ', aSQueakLTableColumnAction field].

	((aSQueakLTableColumnAction action = 'ADD') or: [((aSQueakLTableColumnAction action = 'RENAME') or: [aSQueakLTableColumnAction action = 'MODIFY'])]) ifTrue: [
		colType := SQueakLConnection formatColumnDefn: aSQueakLTableColumnAction.		
		alterTableCommand := alterTableCommand , ' ', colType.
 		].

	ddlError := self executeDDL: alterTableCommand.

	self updateTabCols: aSQueakLTable columnAction: aSQueakLTableColumnAction.
	
	^ddlError 
	! !

!SQueakLConnMYSQL methodsFor: 'table methods' stamp: 'MartinLeo 7/20/2010 23:26'!
alterTable: aSQueakLTable index: aSQueakLIndexAction
	
		"allows you to modify aSQueakLTables indexes, the action is encapsulated within SQueakLIndexAction>>action.
		It is a subclass of aSQueakLIndex
		options are: CREATE DROP
		
		Many DBs do not support MODIFY so it was left out.
		"
	
	|indexCommand prevTblColDef colType ddlError |

	indexCommand := aSQueakLIndexAction action, ' INDEX ', aSQueakLIndexAction indexName, ' ON ', aSQueakLTable schemaName, '.', aSQueakLTable table. 

	(aSQueakLIndexAction action = 'CREATE') ifTrue: [
		indexCommand := indexCommand , ' ('.
		((aSQueakLIndexAction indexColumns) asSortedCollection: [:c1 :c2 | c1 index <= c2 index]) do: [ :aSQueakLIndexColumn | indexCommand := indexCommand, aSQueakLIndexColumn column, ','].		
			indexCommand := (indexCommand allButLast: 1), ')'.
	].

	ddlError := self executeDDL: indexCommand.

	self updateTabCols: aSQueakLTable indexAction: aSQueakLIndexAction.
	
	^ddlError
	! !

!SQueakLConnMYSQL methodsFor: 'instance release' stamp: 'MartinLeo 7/29/2010 22:44'!
connect: aHost port: aPort database: aDatabase userName: aUserName userPassword: aUserPassword
	"This method instatiates a Mysql type connection."
	
	|connectionSettings|

	connectionSettings := super connect: aHost port: aPort database: aDatabase userName: aUserName userPassword: aUserPassword.
	
	connection := DBXConnection platform: DBXMySQLPlatform new settings: connectionSettings.! !

!SQueakLConnMYSQL methodsFor: 'schema methods' stamp: 'MartinLeo 7/29/2010 22:44'!
createSchema: aSQueakLSchema
	"MYSQL sql for schema screation"
	
	^self executeDDL: 'CREATE DATABASE ', aSQueakLSchema schema
	
! !

!SQueakLConnMYSQL methodsFor: 'table methods' stamp: 'MartinLeo 7/20/2010 23:38'!
createTable: aSQueakLTable
	"create a aSQueakLTable in the database including indexes if any"
	|createTableCommand indexTableCommands ddlError|
	
	createTableCommand := self generateCreateTableStmt: aSQueakLTable.	
	ddlError := self executeDDL: createTableCommand.

	(ddlError error) ifTrue: [^ddlError].
	
	indexTableCommands := self generateCreateIndexStmts: aSQueakLTable.
	indexTableCommands do: [:createIndex | 
		ddlError := self executeDDL: createIndex.
		(ddlError error) ifTrue: [^ddlError]].
	
	^ddlError	

	! !

!SQueakLConnMYSQL methodsFor: 'private' stamp: 'MartinLeo 7/30/2010 23:44'!
discoverColumns: aSQueakLTable
	"These are methods that needs to be implemented because it is called by the template method populateTable.
	
	This method must take a SQueakLTable instance which contains a schema and table name, 
	and populate the column definition of the instance.
	
	User must have rights to query the INFORMATION_SCHEMA schema."

	|errorObject result  columnCollection|

	columnCollection := OrderedCollection new.

	"Populate Columns"

	self open.
	result := connection execute:  'select column_name, data_Type, character_maximum_length, numeric_precision, numeric_scale from',
					' INFORMATION_SCHEMA.columns where table_schema = ''',aSQueakLTable schemaName,''' AND TABLE_NAME = ''', aSQueakLTable table ,''''.

	result rowsDo: [ :dbxRow | |tmpTableColumn|
		tmpTableColumn := SQueakLTableColumn new.
		tmpTableColumn field: (dbxRow valueNamed: 'column_name').
		tmpTableColumn type: (self cleanColumnDataType:(dbxRow valueNamed: 'data_Type')).
		tmpTableColumn charLength: (dbxRow valueNamed: 'character_maximum_length').
		tmpTableColumn precision: (dbxRow valueNamed: 'numeric_precision').
		tmpTableColumn scale: (dbxRow valueNamed: 'numeric_scale').
		columnCollection add: tmpTableColumn.
		].

	self close.
	
	^columnCollection! !

!SQueakLConnMYSQL methodsFor: 'private' stamp: 'MartinLeo 7/29/2010 22:32'!
discoverIndex: aSQueakLTable
	"These are methods that needs to be implemented because it is called by the template method populateTable.
	
	This method must take a SQueakLTable instance which contains a schema and table name, 
	and populate the index definition of the instance.
	
	User must have rights to query the INFORMATION_SCHEMA schema."

	|errorObject result indexNameCollection indexCollection|

	indexNameCollection := Set new.
	indexCollection := OrderedCollection new.
	self open.

	"Populate Names of Indexes"

	result := connection execute: 'SHOW INDEXES FROM ', aSQueakLTable schemaName ,'.', aSQueakLTable table.
	result rowsDo: [ :dbxRow |
		indexNameCollection add: (dbxRow valueNamed: 'Key_name')
		].

	indexNameCollection do: [ :anIndexName | |tmpIndex indexType|
	result := connection execute: 'SHOW INDEXES FROM ', aSQueakLTable schemaName ,'.', aSQueakLTable table, ' WHERE KEY_NAME = ''', anIndexName ,''''.

	tmpIndex := SQueakLIndex new.
	tmpIndex indexName: anIndexName.
	tmpIndex tableName: aSQueakLTable table.
	result rowsDo: [ :dbxRow | |tmpIDXColumn|
		tmpIDXColumn := SQueakLIndexColumn  new.
		tmpIDXColumn index: (dbxRow valueNamed: 'Seq_in_index').
		tmpIDXColumn column: (dbxRow valueNamed: 'Column_name').
		indexType := (dbxRow valueNamed: 'Index_type').
		tmpIndex addDetail: tmpIDXColumn.
		].
	tmpIndex indexType: indexType. 
	indexCollection add: tmpIndex.
	].

	self close.

	^indexCollection ! !

!SQueakLConnMYSQL methodsFor: 'schema methods' stamp: 'MartinLeo 7/13/2010 21:55'!
dropSchema: aSQueakLSchema
	^self executeDDL: 'DROP DATABASE ', aSQueakLSchema schema
	
! !

!SQueakLConnMYSQL methodsFor: 'table methods' stamp: 'MartinLeo 7/20/2010 23:39'!
dropTable: aSQueakLTable
	^self executeDDL: 'DROP TABLE ', aSQueakLTable schemaName ,'.' , aSQueakLTable table , ' CASCADE'
	
		! !

!SQueakLConnMYSQL methodsFor: 'private' stamp: 'MartinLeo 7/29/2010 22:44'!
generateCreateIndexStmts: aSQueakLTable
	"Given a SQueakLTable instance, this method generates the necessary MYSQL SQL calls to create its indexes."

	|indexTableCommands|
	
	indexTableCommands := OrderedCollection new.
	
	aSQueakLTable indexes do: [:aSQueakLIndex | |idxCreate|
		idxCreate := 'CREATE INDEX ', aSQueakLIndex indexName , ' ON ', aSQueakLTable schemaName , '.', aSQueakLTable table ,'('.
		
		((aSQueakLIndex indexColumns) asSortedCollection: [:c1 :c2 | c1 index <= c2 index]) do: [ :aSQueakLIndexColumn |
				idxCreate := idxCreate, aSQueakLIndexColumn column, ','.
			].
		idxCreate := (idxCreate allButLast: 1), ')'.
		indexTableCommands add: idxCreate.
		 ].
	
	^indexTableCommands ! !

!SQueakLConnMYSQL methodsFor: 'private' stamp: 'MartinLeo 7/30/2010 23:17'!
generateCreateTableStmt: aSQueakLTable
	"Given a SQueakLTable instance, this method generates the necessary MYSQL SQL calls to define a table and its columns."

	|createTableCommand|
	
	createTableCommand := 'CREATE TABLE ', aSQueakLTable schemaName , '.', aSQueakLTable table ,'('.
	
	aSQueakLTable columns do: [:aColumn| |colType colTypeDef|
		colType := SQueakLConnection formatColumnDefn: aColumn.		
		createTableCommand := createTableCommand , aColumn field, ' ', colType, ', '.
		].
	
	createTableCommand := createTableCommand allButLast: 2.
	createTableCommand := createTableCommand, ')'.
	
	^createTableCommand! !

!SQueakLConnMYSQL methodsFor: 'index methods' stamp: 'MartinLeo 7/31/2010 21:34'!
indexes: aSQueakLSchema
	"Returns a details fully populated OrderedCollection  of SQueakLIndex that exists within aSQueakLSchema.
	"

	|errorObject result indexCollection|

	indexCollection := OrderedCollection new.

	self open.
	result := connection execute: 'select distinct table_name, index_name, index_type from information_schema.statistics where table_schema = ''', aSQueakLSchema schema ,''''.

	result rowsDo: [ :dbxRow | | tmpIndex |
		tmpIndex := SQueakLIndex new.
		tmpIndex tableName: (dbxRow valueNamed: 'table_name').
		tmpIndex indexName: (dbxRow valueNamed: 'index_name').
		tmpIndex indexType: (dbxRow valueNamed: 'index_type').
		tmpIndex schemaName: aSQueakLSchema schema.

		indexCollection add: tmpIndex.
		].

	self close.

	indexCollection do: [ :aSQueakLIndex | self populateIndexDetail: aSQueakLIndex].
	
	^indexCollection ! !

!SQueakLConnMYSQL methodsFor: 'private' stamp: 'MartinLeo 7/29/2010 22:11'!
populateIndexDetail: aSQueakLIndex
	"This is a helper method to the indexes: method that takes an index object and populate its details (columns indexed)"
	
	| result |
	
	self open.
	result := self execute: 'SHOW INDEXES FROM ', aSQueakLIndex schemaName ,'.', aSQueakLIndex tableName, ' WHERE KEY_NAME = ''', aSQueakLIndex indexName ,''''.

	result rowsDo: [ :dbxRow | |tmpIDXColumn|
		tmpIDXColumn := SQueakLIndexColumn  new.
		tmpIDXColumn index: (dbxRow valueNamed: 'Seq_in_index').
		tmpIDXColumn column: (dbxRow valueNamed: 'Column_name').
		aSQueakLIndex addDetail: tmpIDXColumn.
		].

	self close
	
! !

!SQueakLConnMYSQL methodsFor: 'schema methods' stamp: 'MartinLeo 7/29/2010 22:11'!
schemas
	"Show all schemas detected by the current user"	
		
	|result schemaCollection|
	
	schemaCollection := OrderedCollection new.
	
	self open.
	result := connection execute: 'show databases'.
	result rowsDo: [ :dbxRow | |tmpSchema|
		tmpSchema := SQueakLSchema named: (dbxRow valueNamed: 'Database') withConnection: self.
		
		schemaCollection add: tmpSchema.
		].
	self close.
	
	^schemaCollection ! !

!SQueakLConnMYSQL methodsFor: 'table methods' stamp: 'MartinLeo 7/29/2010 22:16'!
tables: aSQueakLSchema
	"Retrieves all tables in the provided schema"	
		
	|errorObject result tableCollection|

	tableCollection := OrderedCollection new.
	self open.
	errorObject := self executeDDLOnly: 'USE ', aSQueakLSchema schema.
	errorObject error ifTrue:[self error: (errorObject message)].

	result := connection execute: 'show full tables where table_type = ''BASE TABLE'''.
	result rowsDo: [ :dbxRow | |tmpTable|
		tmpTable := SQueakLTable new.
		tmpTable table: (dbxRow valueAt: 1).
		tmpTable schema: aSQueakLSchema.
		tableCollection add: tmpTable.
		].
	self close.
	
	^tableCollection ! !

!SQueakLConnMYSQL methodsFor: 'view methods' stamp: 'MartinLeo 7/29/2010 22:17'!
views: aSQueakLSchema
	"Retrieves all views in the provided schema"	
	|errorObject result viewCollection|

	viewCollection := OrderedCollection new.
	self open.

	result := connection execute: 'select table_name, cast(view_definition as char) view_def from information_schema.views where table_schema = ''' , aSQueakLSchema schema ,''''.

	result rowsDo: [ :dbxRow | |tmpView|
		tmpView := SQueakLView new.
		tmpView schema: aSQueakLSchema.
		tmpView viewName: (dbxRow valueNamed: 'table_name').
		tmpView viewDef: (self processView: (dbxRow valueNamed: 'view_def')).
		viewCollection add: tmpView.
		].
	self close.
	
	^viewCollection ! !

SQueakLConnection subclass: #SQueakLConnORACLE
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLConnORACLE commentStamp: 'MartinLeo 7/31/2010 00:16' prior: 0!
A SQueakLConnORACLE  is a concrete Strategy for connecting to an Oracle Database.

Instance Variables
!

!SQueakLConnORACLE class methodsFor: 'information methods' stamp: 'MartinLeo 7/10/2010 20:56'!
connectionDatabase
	^'ORACLE'! !

!SQueakLConnORACLE class methodsFor: 'information methods' stamp: 'MartinLeo 7/10/2010 21:59'!
defaultPort
	^'1521'! !

!SQueakLConnORACLE methodsFor: 'table methods' stamp: 'MartinLeo 7/30/2010 23:17'!
alterTable: aSQueakLTable column: aSQueakLTableColumnAction
	"allows you to modify aSQueakLTable, the action is encapsulated within SQueakLTableColumnAction>>action.
		It is a subclass of aSQueakLTableColumn
		options are: ADD RENAME MODIFY(change column type /length) DROP"

	|alterTableCommand prevTblColDef colType ddlError|

	alterTableCommand := 'ALTER TABLE ', aSQueakLTable schemaName ,'.', aSQueakLTable table , ' '.

	(aSQueakLTableColumnAction action = 'RENAME')
	ifTrue: [alterTableCommand :=  alterTableCommand , 'RENAME COLUMN ', aSQueakLTableColumnAction field,' TO ', aSQueakLTableColumnAction newName]
	ifFalse: [
		(aSQueakLTableColumnAction action = 'DROP') ifTrue: [
			alterTableCommand := alterTableCommand , ' DROP COLUMN ' , aSQueakLTableColumnAction field
		]
		ifFalse: [
					alterTableCommand := alterTableCommand , aSQueakLTableColumnAction action, ' ', aSQueakLTableColumnAction field]
		].

	((aSQueakLTableColumnAction action = 'ADD') or: [aSQueakLTableColumnAction action = 'MODIFY']) ifTrue: [
		colType := SQueakLConnection formatColumnDefn: aSQueakLTableColumnAction.		
		alterTableCommand := alterTableCommand , ' ', colType.
 		].

	ddlError := self executeDDL: alterTableCommand.

	self updateTabCols: aSQueakLTable columnAction: aSQueakLTableColumnAction.
	
	^ddlError 
	! !

!SQueakLConnORACLE methodsFor: 'table methods' stamp: 'MartinLeo 7/20/2010 23:36'!
alterTable: aSQueakLTable index: aSQueakLIndexAction
		
		"allows you to modify aSQueakLTables indexes, the action is encapsulated within SQueakLIndexAction>>action.
		It is a subclass of aSQueakLIndex
		options are: CREATE DROP
		
		Many DBs do not support MODIFY so it was left out.
		"

	|indexCommand prevTblColDef colType ddlError|

	indexCommand := aSQueakLIndexAction action, ' INDEX ', aSQueakLTable schemaName , '.', aSQueakLIndexAction indexName.

	(aSQueakLIndexAction action = 'CREATE') ifTrue: [
		indexCommand := indexCommand , ' ON ', aSQueakLTable schemaName, '.', aSQueakLTable table ,' ('.
		((aSQueakLIndexAction indexColumns) asSortedCollection: [:c1 :c2 | c1 index <= c2 index]) do: [ :aSQueakLIndexColumn | indexCommand := indexCommand, aSQueakLIndexColumn column, ','].		
			indexCommand := (indexCommand allButLast: 1), ')'.
	].

	ddlError := self executeDDL: indexCommand.

	self updateTabCols: aSQueakLTable indexAction: aSQueakLIndexAction.
	
	^ddlError
	! !

!SQueakLConnORACLE methodsFor: 'instance release' stamp: 'MartinLeo 7/10/2010 22:17'!
connect: aHost port: aPort database: aDatabase userName: aUserName userPassword: aUserPassword
	|connectionSettings|

	connectionSettings := super connect: aHost port: aPort database: aDatabase userName: aUserName userPassword: aUserPassword.
	
	connection := DBXConnection platform: DBXOraclePlatform new settings: connectionSettings.! !

!SQueakLConnORACLE methodsFor: 'schema methods' stamp: 'MartinLeo 7/13/2010 21:56'!
createSchema: aSQueakLSchema
	"Schemas in Oracle are users. We create a user here, set the password to the username, and set its default tablespace to USERS.
	
	COMMENT: Should we have an advanced version of createSchema to customize everything?

	A directory type mechanism maybe required for the front end, to register # of parameters. We'll see.
	"

	|errorObject|

	errorObject := self executeDDL: 'CREATE USER ', aSQueakLSchema schema, ' IDENTIFIED BY ', aSQueakLSchema schema, 
						' DEFAULT TABLESPACE USERS TEMPORARY TABLESPACE TEMP QUOTA UNLIMITED ON USERS'.
	errorObject error ifTrue:[self error: (errorObject message)].
	
	^self executeDDL: 'GRANT CONNECT, CREATE TABLE, CREATE VIEW, CREATE PROCEDURE, CREATE SYNONYM TO ', aSQueakLSchema schema! !

!SQueakLConnORACLE methodsFor: 'table methods' stamp: 'MartinLeo 7/20/2010 23:38'!
createTable: aSQueakLTable
		"create a aSQueakLTable in the database including indexes if any"
	|createTableCommand indexTableCommands ddlError|

	createTableCommand := self generateCreateTableStmt: aSQueakLTable.	
	ddlError := self executeDDL: createTableCommand.	
	(ddlError error) ifTrue: [^ddlError].
	
	indexTableCommands := self generateCreateIndexStmts: aSQueakLTable.	
	indexTableCommands do: [:createIndex | 	
			ddlError := self executeDDL: createIndex.
			(ddlError error) ifTrue: [^ddlError]].

	^ddlError! !

!SQueakLConnORACLE methodsFor: 'private' stamp: 'MartinLeo 7/30/2010 23:43'!
discoverColumns: aSQueakLTable
	"These are methods that needs to be implemented because it is called by the template method populateTable.
	
	This method must take a SQueakLTable instance which contains a schema and table name, 
	and populate the column definition of the instance.
	"

	|errorObject result  columnCollection|

	columnCollection := OrderedCollection new.

	"Populate Columns"

	self open.
	result := connection execute:  'select COLUMN_NAME, DATA_TYPE, CHAR_LENGTH, DATA_PRECISION, DATA_SCALE from all_tab_cols where owner = ''',
											aSQueakLTable schemaName, ''' and table_name = ''', aSQueakLTable table ,''''.

	result rowsDo: [ :dbxRow | |tmpTableColumn|
		tmpTableColumn := SQueakLTableColumn new.
		tmpTableColumn field: (dbxRow valueNamed: 'COLUMN_NAME').
		tmpTableColumn type: (self cleanColumnDataType: (dbxRow valueNamed: 'DATA_TYPE')).
		tmpTableColumn charLength: (dbxRow valueNamed: 'CHAR_LENGTH').
		tmpTableColumn precision: (dbxRow valueNamed: 'DATA_PRECISION').
		tmpTableColumn scale: (dbxRow valueNamed: 'DATA_SCALE').
		columnCollection add: tmpTableColumn.
		].

	self close.
	
	^columnCollection! !

!SQueakLConnORACLE methodsFor: 'private' stamp: 'MartinLeo 7/29/2010 22:33'!
discoverIndex: aSQueakLTable
	"These are methods that needs to be implemented because it is called by the template method populateTable.
	
	This method must take a SQueakLTable instance which contains a schema and table name, 
	and populate the index definition of the instance."

	|errorObject result indexNameCollection indexCollection|

	indexNameCollection := Dictionary new.
	indexCollection := OrderedCollection new.
	self open.

	"Populate Names of Indexes"

	result := connection execute: 'select index_name, index_type from all_indexes where table_owner = ''', aSQueakLTable schemaName , ''' AND TABLE_NAME = ''', aSQueakLTable table , ''''.
	
	result rowsDo: [ :dbxRow |
		indexNameCollection at: (dbxRow valueNamed: 'INDEX_NAME') put: (dbxRow valueNamed: 'INDEX_TYPE')
		].

	indexNameCollection keysDo: [ :anIndexName | |tmpIndex indexType|
	result := connection execute: 'select column_position, column_name from all_ind_columns where table_owner = ''', aSQueakLTable schemaName, ''' and table_name = ''',
											aSQueakLTable table, ''' and index_name = ''', anIndexName ,''''.

	tmpIndex := SQueakLIndex new.
	tmpIndex indexName: anIndexName.
	tmpIndex tableName: aSQueakLTable table.
	result rowsDo: [ :dbxRow | |tmpIDXColumn|
		tmpIDXColumn := SQueakLIndexColumn  new.
		tmpIDXColumn index: (dbxRow valueNamed: 'COLUMN_POSITION').
		tmpIDXColumn column: (dbxRow valueNamed: 'COLUMN_NAME').
		tmpIndex addDetail: tmpIDXColumn.
		].

	tmpIndex indexType: (indexNameCollection at: anIndexName). 
	indexCollection add: tmpIndex.
	].

	self close.

	^indexCollection ! !

!SQueakLConnORACLE methodsFor: 'schema methods' stamp: 'MartinLeo 7/13/2010 21:56'!
dropSchema: aSQueakLSchema
	^self executeDDL: 'DROP USER ', aSQueakLSchema schema, ' CASCADE'.
	
! !

!SQueakLConnORACLE methodsFor: 'table methods' stamp: 'MartinLeo 7/20/2010 23:39'!
dropTable: aSQueakLTable
	
	|tableDropCommand|

	tableDropCommand := 'DROP TABLE ', aSQueakLTable schemaName , '.', aSQueakLTable table , ' CASCADE CONSTRAINTS PURGE'.

	^self executeDDL: tableDropCommand 
	
		! !

!SQueakLConnORACLE methodsFor: 'private' stamp: 'MartinLeo 7/29/2010 22:30'!
generateCreateIndexStmts: aSQueakLTable

	"Given a SQueakLTable instance, this method generates the necessary Oracle SQL calls to create its indexes."
	
	|indexTableCommands|
	
	indexTableCommands := OrderedCollection new.
	
	aSQueakLTable indexes do: [:aSQueakLIndex | |idxCreate|
		idxCreate := 'CREATE INDEX ', aSQueakLTable schemaName ,'.', aSQueakLIndex indexName , ' ON ', aSQueakLTable schemaName,'.', aSQueakLTable table ,'('.
		
		((aSQueakLIndex indexColumns) asSortedCollection: [:c1 :c2 | c1 index <= c2 index]) do: [ :aSQueakLIndexColumn |
				idxCreate := idxCreate, aSQueakLIndexColumn column, ','.
			].
		idxCreate := (idxCreate allButLast: 1), ')'.
		indexTableCommands add: idxCreate.
		 ].
	
	^indexTableCommands ! !

!SQueakLConnORACLE methodsFor: 'private' stamp: 'MartinLeo 7/30/2010 23:17'!
generateCreateTableStmt: aSQueakLTable
	"Given a SQueakLTable instance, this method generates the necessary Oracle SQL calls to define a table and its columns."

	|createTableCommand|
	
	createTableCommand := 'CREATE TABLE ', aSQueakLTable schemaName ,'.', aSQueakLTable table ,'('.
	
	aSQueakLTable columns do: [:aColumn| |colType colTypeDef|
		colType := colType := SQueakLConnection formatColumnDefn: aColumn.				
		createTableCommand := createTableCommand , aColumn field, ' ', colType, ', '.
		].
	
	createTableCommand := createTableCommand allButLast: 2.
	createTableCommand := createTableCommand, ')'.
	
	^createTableCommand! !

!SQueakLConnORACLE methodsFor: 'index methods' stamp: 'MartinLeo 7/31/2010 21:34'!
indexes: aSQueakLSchema
	"Returns a details fully populated OrderedCollection  of SQueakLIndex that exists within aSQueakLSchema.
	"

	|errorObject result indexCollection|

	indexCollection := OrderedCollection new.
	self open.

	result := connection execute: 'select table_name, index_type, index_name from all_indexes where table_owner = ''', aSQueakLSchema schema ,''''.
	result rowsDo: [ :dbxRow | |tmpIndex|
		tmpIndex := SQueakLIndex new.
		tmpIndex tableName: (dbxRow valueNamed: 'TABLE_NAME').
		tmpIndex indexName: (dbxRow valueNamed: 'INDEX_NAME').
		tmpIndex indexType: (dbxRow valueNamed: 'INDEX_TYPE').
		tmpIndex schemaName: aSQueakLSchema schema.

		indexCollection add: tmpIndex.
		].
	self close.
	
	indexCollection do: [ :aSQueakLIndex | self populateIndexDetail: aSQueakLIndex].
	
	^indexCollection 
	



 ! !

!SQueakLConnORACLE methodsFor: 'view methods' stamp: 'MartinLeo 7/29/2010 22:29'!
populateIndexDetail: aSQueakLIndex
	"This is a helper method to the indexes: method that takes an index object and populate its details (columns indexed)"

	| result |
	
	self open.

	result := connection execute: 'select column_position, column_name from all_ind_columns where table_owner = ''', aSQueakLIndex schemaName, ''' and table_name = ''',
											aSQueakLIndex tableName, ''' and index_name = ''', aSQueakLIndex name ,''''.

	result rowsDo: [ :dbxRow | |tmpIDXColumn|
		tmpIDXColumn := SQueakLIndexColumn  new.
		tmpIDXColumn index: (dbxRow valueNamed: 'COLUMN_POSITION').
		tmpIDXColumn column: (dbxRow valueNamed: 'COLUMN_NAME').
		aSQueakLIndex addDetail: tmpIDXColumn.
		].

	self close
	! !

!SQueakLConnORACLE methodsFor: 'schema methods' stamp: 'MartinLeo 7/29/2010 22:29'!
schemas
	"Show all schemas detected by the current user"	
		
	|result schemaCollection|
	
	schemaCollection := OrderedCollection new.
	
	self open.
	result := connection execute: 'select username from all_users where username not in (''OUTLN'',''SYSTEM'',''DIP'',''ORACLE_OCM'',''DBSNMP'',''APPQOSSYS'',''EXFSYS'',''CTXSYS'',''ANONYMOUS'',''OLAPSYS'',''ORDSYS'',
''ORDDATA'',''ORDPLUGINS'',''SI_INFORMTN_SCHEMA'',''MDSYS'',''MDDATA'',''SPATIAL_WFS_ADMIN_USR'',''SPATIAL_CSW_ADMIN_USR'',''SYSMAN'',''MGMT_VIEW'',''FLOWS_FILES'',
''APEX_PUBLIC_USER'',''APEX_030200'',''OWBSYS_AUDIT'',''OWBSYS'',''SYS'',''WMSYS'',''XS$NULL'',''XDB'')'.
	result rowsDo: [ :dbxRow | |tmpSchema|
		tmpSchema := SQueakLSchema named: (dbxRow valueNamed: 'USERNAME') withConnection: self.

		schemaCollection add: tmpSchema.
		].
	self close.
	
	^schemaCollection ! !

!SQueakLConnORACLE methodsFor: 'table methods' stamp: 'MartinLeo 7/29/2010 22:29'!
tables: aSQueakLSchema
	"Retrieves all tables in the provided schema"	
		
	|result tableCollection|

	tableCollection := OrderedCollection new.

	self open.
	result := connection execute: 'select table_name from all_tables where owner = ''',aSQueakLSchema schema,''''.
	result rowsDo: [ :dbxRow | |tmpTable|
		tmpTable := SQueakLTable new.
		tmpTable table: (dbxRow valueNamed: 'TABLE_NAME').
		tmpTable schema: aSQueakLSchema.
		tableCollection add: tmpTable.
		].
	self close.
	
	^tableCollection ! !

!SQueakLConnORACLE methodsFor: 'view methods' stamp: 'MartinLeo 7/29/2010 22:24'!
views: aSQueakLSchema
	"Retrieves all views in the provided schema.

	View definitions are usually represented as CLOBS / RAW in a database, 
	OpenDBX cannot read clobs, need to convert to a varchar / char first.
	
	However ORACLE cannot do a direct CAST to VARCHAR. so a temporary staging table is needed to created to hold intermin results.
	
	Here Views are stored as LONG, we convert it to CLOBs first before casting it to a VARCHAR.
	
	WARNING:
	1. If the user used to connect does not have rights to create a table, we assume that there are no views in the schema,
			because it is opaque to us.
	2. Views can be very very long, but we can only cast the first 4000 characters of it. Some complicated view defnitions
	maybe truncated.
	"	
		
	|errorObject result viewCollection |

	viewCollection := OrderedCollection new.

	self open.
	[|createStmt| 
	createStmt := 'CREATE TABLE ', aSQueakLSchema schema , '.ZZZ_SQLITE_TMP (NAME VARCHAR2(500), STRING CLOB)'.
	connection execute: createStmt.]
	on: Exception
	do: ["nothing"].


	[self close]
	on: Exception
	do: ["nothing"].

	self open.

	[connection execute: 'insert into ', aSQueakLSchema schema , '.ZZZ_SQLITE_TMP select view_name, to_lob(text) from all_views where owner = ''', aSQueakLSchema schema, '''']
	on: Exception
	do: ["nothing"].

	[self close]
	on: Exception
	do: ["nothing"].

	self open.
	[result := connection execute: 'select name, dbms_lob.substr(string,4000) string from ', aSQueakLSchema schema, '.ZZZ_SQLITE_TMP'.
	
	result rowsDo: [ :dbxRow | |tmpView|
		tmpView := SQueakLView new.
		tmpView schema: aSQueakLSchema.
		tmpView viewName: (dbxRow valueNamed: 'name').
		tmpView viewDef: (self processView: (dbxRow valueNamed: 'string')).
		viewCollection add: tmpView.
		].
	] on: Exception do: ["nothing"].

	[self close] on: Exception
	do: ["nothing"].

	self open.
	[connection execute: 'DROP TABLE ', aSQueakLSchema schema, '.ZZZ_SQLITE_TMP']
	on: Exception do: ["nothing"].

	[self close] on: Exception do:["nothing"].
	
	^viewCollection ! !

SQueakLConnection subclass: #SQueakLConnPOSTGRES
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLConnPOSTGRES commentStamp: 'MartinLeo 7/31/2010 00:17' prior: 0!
A SQueakLConnPOSTGRES is a concrete Strategy for connecting to an Oracle Database.

Instance Variables
!

!SQueakLConnPOSTGRES class methodsFor: 'information methods' stamp: 'MartinLeo 7/10/2010 20:56'!
connectionDatabase
	^'POSTGRES'! !

!SQueakLConnPOSTGRES class methodsFor: 'information methods' stamp: 'MartinLeo 7/10/2010 21:59'!
defaultPort
	^'5432'! !

!SQueakLConnPOSTGRES methodsFor: 'table methods' stamp: 'MartinLeo 7/30/2010 23:17'!
alterTable: aSQueakLTable column: aSQueakLTableColumnAction
		"allows you to modify aSQueakLTable, the action is encapsulated within SQueakLTableColumnAction>>action.
		It is a subclass of aSQueakLTableColumn
		options are: ADD RENAME MODIFY(change column type /length) DROP"

	|alterTableCommand prevTblColDef colType ddlError|

	alterTableCommand := 'ALTER TABLE ', aSQueakLTable schemaName ,'.', aSQueakLTable table , ' '.

	(aSQueakLTableColumnAction action = 'RENAME')
	ifTrue: [alterTableCommand :=  alterTableCommand , 'RENAME COLUMN ', aSQueakLTableColumnAction field,' TO ', aSQueakLTableColumnAction newName]
	ifFalse: [
		(aSQueakLTableColumnAction action = 'DROP') ifTrue: [
			alterTableCommand := alterTableCommand , ' DROP COLUMN ' , aSQueakLTableColumnAction field
		]
		ifFalse: [
			(aSQueakLTableColumnAction action = 'MODIFY') 
				ifTrue: [alterTableCommand := alterTableCommand ,' ALTER COLUMN ', aSQueakLTableColumnAction field, ' TYPE ' ]			
				ifFalse: [alterTableCommand := alterTableCommand , aSQueakLTableColumnAction action, ' ', aSQueakLTableColumnAction field]
		]
	].
	((aSQueakLTableColumnAction action = 'ADD') or: [aSQueakLTableColumnAction action = 'MODIFY']) ifTrue: [
		colType := SQueakLConnection formatColumnDefn: aSQueakLTableColumnAction.		
		alterTableCommand := alterTableCommand , ' ', colType.
 		].

	ddlError := self executeDDL: alterTableCommand.

	self updateTabCols: aSQueakLTable columnAction: aSQueakLTableColumnAction.

	^ddlError 
	! !

!SQueakLConnPOSTGRES methodsFor: 'table methods' stamp: 'MartinLeo 7/20/2010 23:41'!
alterTable: aSQueakLTable index: aSQueakLIndexAction
		
		"allows you to modify aSQueakLTables indexes, the action is encapsulated within SQueakLIndexAction>>action.
		It is a subclass of aSQueakLIndex
		options are: CREATE DROP
		
		Many DBs do not support MODIFY so it was left out.
		"

	|indexCommand prevTblColDef colType ddlError|

	(aSQueakLIndexAction action = 'CREATE') ifTrue: [
	indexCommand := 'CREATE INDEX ', aSQueakLIndexAction indexName
	]
	ifFalse: [
			indexCommand := 'DROP INDEX ', aSQueakLTable schemaName ,'.', aSQueakLIndexAction indexName
		].

	(aSQueakLIndexAction action = 'CREATE') ifTrue: [
		indexCommand := indexCommand , ' ON ', aSQueakLTable schemaName , '.', aSQueakLTable table ,' ('.
		((aSQueakLIndexAction indexColumns) asSortedCollection: [:c1 :c2 | c1 index <= c2 index]) do: [ :aSQueakLIndexColumn | indexCommand := indexCommand, aSQueakLIndexColumn column, ','].		
			indexCommand := (indexCommand allButLast: 1), ')'.
	]
	ifFalse: [indexCommand := indexCommand, ' CASCADE'].

	ddlError := self executeDDL: indexCommand.

	self updateTabCols: aSQueakLTable indexAction: aSQueakLIndexAction.
	
	^ddlError 
	! !

!SQueakLConnPOSTGRES methodsFor: 'connection methods' stamp: 'MartinLeo 7/10/2010 22:18'!
connect: aHost port: aPort database: aDatabase userName: aUserName userPassword: aUserPassword
	|connectionSettings|

	connectionSettings := super connect: aHost port: aPort database: aDatabase userName: aUserName userPassword: aUserPassword.
	
	connection := DBXConnection platform: DBXPostgresPlatform new settings: connectionSettings.! !

!SQueakLConnPOSTGRES methodsFor: 'schema methods' stamp: 'MartinLeo 7/13/2010 21:57'!
createSchema: aSQueakLSchema
	^self executeDDL: 'CREATE SCHEMA ', aSQueakLSchema schema
	
! !

!SQueakLConnPOSTGRES methodsFor: 'table methods' stamp: 'MartinLeo 7/20/2010 23:43'!
createTable: aSQueakLTable
		"create a aSQueakLTable in the database including indexes if any"
	|createTableCommand indexTableCommands ddlError|

	createTableCommand := self generateCreateTableStmt: aSQueakLTable.	
	ddlError := self executeDDL: createTableCommand.
	(ddlError error) ifTrue: [^ddlError].
	
	indexTableCommands := self generateCreateIndexStmts: aSQueakLTable.
	indexTableCommands do: [:createIndex | 
		ddlError := self executeDDL: createIndex.
		(ddlError error) ifTrue: [^ddlError]].
		
	^ddlError! !

!SQueakLConnPOSTGRES methodsFor: 'private' stamp: 'MartinLeo 7/30/2010 23:45'!
discoverColumns: aSQueakLTable
	"These are methods that needs to be implemented because it is called by the template method populateTable.
	
	This method must take a SQueakLTable instance which contains a schema and table name, 
	and populate the column definition of the instance.
	
	Postgres does not store original table definitions inside a data dictionary, instead it has a C function that
	given a datatype and how much space the data type uses, computes the column definition on the fly.
	
	We cannot directly query the data dictionary here, instead, we pull out column definitions and parse them to extract the definitions."

	|errorObject result  columnCollection re executionString|

	re := '[\w\W]*\((.*)\)' asRegexIgnoringCase.
	columnCollection := OrderedCollection new.

	"Populate Columns"
	executionString := 'SELECT cast(a.attname as varchar), cast(ty.typname as varchar), cast(pg_catalog.format_type(a.atttypid, a.atttypmod) as varchar) FROM pg_catalog.pg_attribute a join pg_catalog.pg_type ty on (ty.oid = a.atttypid) ',
'WHERE a.attrelid IN (SELECT c.oid FROM pg_catalog.pg_class c WHERE c.relname =''', aSQueakLTable table ,''' AND c.relnamespace in (select oid from pg_namespace where nspname = ''', aSQueakLTable schemaName  ,''') ',
') AND a.attnum > 0 AND NOT a.attisdropped order by a.attnum'.
	
	self open.
	result := connection execute: executionString.


	result rowsDo: [ :dbxRow | |tmpTableColumn i|
		tmpTableColumn := SQueakLTableColumn new.
		tmpTableColumn field: (dbxRow valueNamed: 'attname').
		tmpTableColumn type: (self cleanColumnDataType:(dbxRow valueNamed: 'typname')).

		(re matchesPrefix: (dbxRow valueNamed: 'format_type'))
			ifTrue: [ |array|
				array := (re subexpression: 2) subStrings: '$,'.
				(array size = 1) ifTrue: [
					tmpTableColumn charLength: (array at: 1).
					tmpTableColumn precision: nil.
					tmpTableColumn scale: nil.
					]
					ifFalse: [
					tmpTableColumn charLength: nil.
					tmpTableColumn precision: (array at: 1).
					tmpTableColumn scale: (array at: 2).						
						]
				]
			ifFalse: [
				tmpTableColumn charLength: nil.
				tmpTableColumn precision: nil.
				tmpTableColumn scale: nil.
				].

		columnCollection add: tmpTableColumn.
		].

	self close.
	
	^columnCollection! !

!SQueakLConnPOSTGRES methodsFor: 'private' stamp: 'MartinLeo 7/29/2010 22:41'!
discoverIndex: aSQueakLTable
	"These are methods that needs to be implemented because it is called by the template method populateTable.
	
	This method must take a SQueakLTable instance which contains a schema and table name, 
	and populate the index definition of the instance.
	"

	|errorObject result indexNameCollection indexCollection i|

	indexNameCollection := Dictionary new.
	indexCollection := OrderedCollection new.
	self open.
											
	result := connection execute: 'select cast(idx.relname as varchar), cast(amname as varchar) from pg_class idx join pg_am on (pg_am.oid = idx.relam) join pg_index on (pg_index.indexrelid = idx.oid) ',
											'join pg_class parent_table on (parent_table.oid = pg_index.indrelid) where idx.relnamespace in (select oid from pg_namespace where ',
											' nspname = ''', aSQueakLTable schemaName ,''') and parent_table.relname = ''', aSQueakLTable table ,''''.
	
	result rowsDo: [ :dbxRow |
		indexNameCollection at: (dbxRow valueNamed: 'relname') put: (dbxRow valueNamed: 'amname')
		].


	indexNameCollection keysDo: [ :anIndexName | |tmpIndex|
	result := connection execute: 'SELECT d.indisprimary, d.indisunique, cast(a.attname as varchar) FROM pg_class t, pg_class i, pg_index d, pg_attribute a, generate_series(0, 9) AS c(i) ',
										'WHERE i.relkind = ''i'' AND d.indexrelid = i.oid AND d.indisprimary = ''f'' AND t.oid = d.indrelid AND t.relname = ''', aSQueakLTable table ,''' ',
										'AND i.relnamespace IN (SELECT oid FROM pg_namespace WHERE nspname = ''', aSQueakLTable schemaName ,''' ) AND a.attrelid = t.oid AND d.indkey[c.i]=a.attnum ',
										'AND i.relname = ''', anIndexName ,''' ORDER BY i.relname, c.i'.

	tmpIndex := SQueakLIndex new.
	tmpIndex indexName: anIndexName.
	tmpIndex tableName: aSQueakLTable table.
	i := 1.
	result rowsDo: [ :dbxRow | |tmpIDXColumn|
		tmpIDXColumn := SQueakLIndexColumn  new.
		tmpIDXColumn index: i.
		tmpIDXColumn column: (dbxRow valueNamed: 'attname').
		i := i + 1.
		tmpIndex addDetail: tmpIDXColumn.
		].

	tmpIndex indexType: (indexNameCollection at: anIndexName). 
	indexCollection add: tmpIndex.
	].

	self close.

	^indexCollection ! !

!SQueakLConnPOSTGRES methodsFor: 'schema methods' stamp: 'MartinLeo 7/13/2010 21:57'!
dropSchema: aSQueakLSchema
	^self executeDDL: 'DROP SCHEMA ', aSQueakLSchema schema, ' CASCADE'.
	
! !

!SQueakLConnPOSTGRES methodsFor: 'table methods' stamp: 'MartinLeo 7/20/2010 23:43'!
dropTable: aSQueakLTable
	^self executeDDL: 'DROP TABLE ', aSQueakLTable schemaName ,'.', aSQueakLTable table , ' CASCADE'
	
		! !

!SQueakLConnPOSTGRES methodsFor: 'private' stamp: 'MartinLeo 7/29/2010 22:42'!
generateCreateIndexStmts: aSQueakLTable
	"Given a SQueakLTable instance, this method generates the necessary POSTGRES SQL calls to create its indexes."

	|indexTableCommands|
	
	indexTableCommands := OrderedCollection new.
	
	aSQueakLTable indexes do: [:aSQueakLIndex | |idxCreate|
		idxCreate := 'CREATE INDEX ', aSQueakLIndex indexName , ' ON ', aSQueakLTable schemaName,'.', aSQueakLTable table ,'('.
		
		((aSQueakLIndex indexColumns) asSortedCollection: [:c1 :c2 | c1 index <= c2 index]) do: [ :aSQueakLIndexColumn |
				idxCreate := idxCreate, aSQueakLIndexColumn column, ','.
			].
		idxCreate := (idxCreate allButLast: 1), ')'.
		indexTableCommands add: idxCreate.
		 ].
	
	^indexTableCommands ! !

!SQueakLConnPOSTGRES methodsFor: 'private' stamp: 'MartinLeo 7/30/2010 23:17'!
generateCreateTableStmt: aSQueakLTable
	"Given a SQueakLTable instance, this method generates the necessary POSTGRES SQL calls to create its indexes."

	|createTableCommand|
	
	createTableCommand := 'CREATE TABLE ', aSQueakLTable schemaName ,'.', aSQueakLTable table ,'('.
	
	aSQueakLTable columns do: [:aColumn| |colType colTypeDef|
		colType := colType := SQueakLConnection formatColumnDefn: aColumn.		
		createTableCommand := createTableCommand , aColumn field, ' ', colType, ', '.
		].
	
	createTableCommand := createTableCommand allButLast: 2.
	createTableCommand := createTableCommand, ')'.
	
	^createTableCommand! !

!SQueakLConnPOSTGRES methodsFor: 'index methods' stamp: 'MartinLeo 7/31/2010 21:34'!
indexes: aSQueakLSchema
	"Returns a details fully populated OrderedCollection  of SQueakLIndex that exists within aSQueakLSchema.
	"

	|errorObject result indexCollection|

	indexCollection := OrderedCollection new.
	self open.

	result := connection execute: 'select cast(parent_table.relname as varchar) as "table", cast(idx.relname as varchar) as "index", cast(amname as varchar) as "type" ',
	'from pg_class idx join pg_am on (pg_am.oid = idx.relam) join pg_index on (pg_index.indexrelid = idx.oid) ',
	'join pg_class parent_table on (parent_table.oid = pg_index.indrelid) where idx.relnamespace in (select oid from pg_namespace where ',
	'nspname = ''', aSQueakLSchema schema ,''')'.

	result rowsDo: [ :dbxRow | |tmpIndex|
		tmpIndex := SQueakLIndex new.
		tmpIndex tableName: (dbxRow valueNamed: 'table').
		tmpIndex indexName: (dbxRow valueNamed: 'index').
		tmpIndex indexType: (dbxRow valueNamed: 'type').
		tmpIndex schemaName: aSQueakLSchema schema.

		indexCollection add: tmpIndex.
		].
	self close.
	
	indexCollection do: [ :aSQueakLIndex | self populateIndexDetail: aSQueakLIndex].
	
	^indexCollection 
	



 ! !

!SQueakLConnPOSTGRES methodsFor: 'table methods' stamp: 'MartinLeo 7/29/2010 22:43'!
populateIndexDetail: aSQueakLIndex

	"This is a helper method to the indexes: method that takes an index object and populate its details (columns indexed)"
	| result i |
	
	self open.

	result := connection execute: 'SELECT d.indisprimary, d.indisunique, cast(a.attname as varchar) FROM pg_class t, pg_class i, pg_index d, pg_attribute a, generate_series(0, 9) AS c(i) ',
										'WHERE i.relkind = ''i'' AND d.indexrelid = i.oid AND d.indisprimary = ''f'' AND t.oid = d.indrelid AND t.relname = ''', aSQueakLIndex tableName ,''' ',
										'AND i.relnamespace IN (SELECT oid FROM pg_namespace WHERE nspname = ''', aSQueakLIndex schemaName ,''' ) AND a.attrelid = t.oid AND d.indkey[c.i]=a.attnum ',
										'AND i.relname = ''', aSQueakLIndex indexName ,''' ORDER BY i.relname, c.i'.

	i := 1.

	result rowsDo: [ :dbxRow | |tmpIDXColumn|
		tmpIDXColumn := SQueakLIndexColumn  new.
		tmpIDXColumn index: i.
		tmpIDXColumn column: (dbxRow valueNamed: 'attname').
		aSQueakLIndex addDetail: tmpIDXColumn.
		i := i + 1.
		].

	self close
	! !

!SQueakLConnPOSTGRES methodsFor: 'schema methods' stamp: 'PeterWubbels 7/17/2010 16:24'!
schemas
	|result schemaCollection|
	
	schemaCollection := OrderedCollection new.
	
	self open.
	result := connection execute: 'select cast(nspname as varchar) from pg_catalog.pg_namespace where nspname not in (''pg_catalog'',''pg_toast'')'.
	result rowsDo: [ :dbxRow | |tmpSchema|
		tmpSchema := SQueakLSchema named: (dbxRow valueNamed: 'nspname') withConnection: self.
				
		"tmpSchema := SQueakLSchema new.
		tmpSchema schema: (dbxRow valueNamed: 'nspname')."
		
		schemaCollection add: tmpSchema.
		].
	self close.
	
	^schemaCollection ! !

!SQueakLConnPOSTGRES methodsFor: 'table methods' stamp: 'MartinLeo 7/29/2010 22:44'!
tables: aSQueakLSchema
	"Retrieves all tables in the provided schema"	

	|result tableCollection executeStmt|

	tableCollection := OrderedCollection new.

	executeStmt := 'SELECT cast(c.relname as varchar) FROM pg_catalog.pg_class c JOIN pg_catalog.pg_roles r ON r.oid = c.relowner ',
									     'LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace WHERE c.relkind IN (''r'','''') ',
      									  'AND n.nspname NOT IN (''pg_catalog'', ''pg_toast'') AND n.nspname = ''', aSQueakLSchema schema ,''''.
	
	self open.
	result := connection execute: executeStmt.

	result rowsDo: [ :dbxRow | |tmpTable|
		tmpTable := SQueakLTable new.
		tmpTable table: (dbxRow valueNamed: 'relname').
		tmpTable schema: aSQueakLSchema.
		tableCollection add: tmpTable.
		].
	self close.
	
	^tableCollection ! !

!SQueakLConnPOSTGRES methodsFor: 'view methods' stamp: 'MartinLeo 7/29/2010 22:44'!
views: aSQueakLSchema
	"Retrieves all views in the provided schema"	

	|errorObject result viewCollection|

	viewCollection := OrderedCollection new.
	self open.

	result := connection execute: 'select cast(viewname as varchar), cast(definition as varchar) from pg_views where schemaname = ''', aSQueakLSchema schema ,''''.

	result rowsDo: [ :dbxRow | |tmpView|
		tmpView := SQueakLView new.
		tmpView schema: aSQueakLSchema.
		tmpView viewName: (dbxRow valueNamed: 'viewname').
		tmpView viewDef: (self processView: ((dbxRow valueNamed: 'definition') allButLast: 1)).
		viewCollection add: tmpView.
		].
	self close.
	
	^viewCollection ! !

!SQueakLConnection class methodsFor: 'test' stamp: 'MartinLeo 7/31/2010 11:56'!
accepts: aDatabaseName
	"Checks whether the concrete strategy can create aDatabaseName type of connection,
	connectionDatabase will have to be defined in the concrete strategy."
	
	^aDatabaseName = self connectionDatabase ! !

!SQueakLConnection class methodsFor: 'connection factory' stamp: 'MartinLeo 7/31/2010 11:58'!
create: aDatabaseType
	" *** IMPORTANT ***
		This class method is the starting point of all database operations.
		A request is sent to this method to start the connection workflow.
		
		eg. SQueakLConnection create: 'ORACLE'.
	"

	|databaseObject|

	databaseObject := self databaseFactory: aDatabaseType.
	
	databaseObject
		ifNil: [^self error: 'Unknown database type.'].

	^databaseObject new! !

!SQueakLConnection class methodsFor: 'connection factory' stamp: 'MartinLeo 7/31/2010 12:03'!
databaseFactory: databaseName
	"This factory method performs the work of:
	1. validating if a concrete strategy can support a certain connection type
	2. then create then responsible connection object"

	|connectionObject|
	connectionObject := SQueakLConnectionTypes
		detect: [:connectionType | connectionType accepts: databaseName]
		ifNone: [^nil].
	^connectionObject! !

!SQueakLConnection class methodsFor: 'table methods' stamp: 'MartinLeo 7/30/2010 23:16'!
formatColumnDefn: aColumn
	"In databases, a data type is represented as 
	1. Name
	2. Data Type
	3. Character Length (populated if the field holds strings, nil otherwise)
	4. Precision (does not apply to strings)
	5. Scale (whats after the decimal place, applies to floats / reals / decimals only)
	
	So given a table column, we need to format it into a form the database understands, 
	this method will detect the data type and convert appropriately.
	"
	
	|colType|
	
		colType := aColumn type asUppercase. "some databases return types in lowercase"

		((colType = 'INTEGER') or: [ (colType = 'DATE') or: [colType = 'TIMESTAMP']]) ifFalse: [
					(colType = 'VARCHAR' or: [colType = 'CHAR']) ifTrue: [colType := colType, '(', aColumn charLength  asString,')'].
					(colType = 'FLOAT') ifTrue: [ colType := colType, '(', aColumn precision asString ,')'].
					(colType = 'DECIMAL') ifTrue: [colType := colType, '(', aColumn precision asString,',', aColumn scale asString,')'].
			].		
			
	^colType! !

!SQueakLConnection class methodsFor: 'initialization' stamp: 'MartinLeo 8/2/2010 10:47'!
initialize
	"
	This method should only be executed once in its life,
	to set up the default connection objects for SQueakLConnection and initialize the data mapping table.
	
	Subsequent registration should send:
		 SQueakLConnection register: SQueakLConnNEWDATABASE
		 SQueakLConnection newType: aTypeSymbol mapTo: aGenericTypeString
	"

	SQueakLConnectionTypes :=  OrderedCollection new.
	SQueakLConnectionTypes add: SQueakLConnMYSQL.
	SQueakLConnectionTypes add: SQueakLConnORACLE.
	SQueakLConnectionTypes add: SQueakLConnPOSTGRES.

	DataTypeMap := Dictionary new.
	
	"The first 7 lines are the generic db types themselves.
	The behavior is this: Provide me with a generic type and you get the same type back."
	DataTypeMap at: #'INTEGER' put: 'INTEGER'.
	DataTypeMap at: #'DECIMAL' put: 'DECIMAL'.
	DataTypeMap at: #'VARCHAR' put: 'VARCHAR'.
	DataTypeMap at: #'CHAR' put: 'CHAR'.
	DataTypeMap at: #'DATE' put: 'DATE'.
	DataTypeMap at: #'TIMESTAMP' put: 'TIMESTAMP'.
	DataTypeMap at: #'FLOAT' put: 'FLOAT'.
	
	"These types are for MySQL / Oracle / PostGRESQL
	Many more types were reviewed and deemed unmappable."

	"MYSQL Mappings"
	DataTypeMap at: #'DOUBLE' put: 'FLOAT'.

	"MYSQL and POSTGRES Mappings"
	DataTypeMap at: #'INT' put: 'INTEGER'.
	DataTypeMap at: #'BIT' put: 'INTEGER'.
	DataTypeMap at: #'TINYINT' put: 'INTEGER'.
	DataTypeMap at: #'SMALLINT' put: 'INTEGER'.
	DataTypeMap at: #'MEDIUMINT' put: 'INTEGER'.
	DataTypeMap at: #'BIGINT' put: 'INTEGER'.
	DataTypeMap at: #'NUMERIC' put: 'DECIMAL'.

	"Oracle Mappings"
	DataTypeMap at: #'VARCHAR2' put: 'VARCHAR'.
	DataTypeMap at: #'NVARCHAR2' put: 'VARCHAR'.
	DataTypeMap at: #'NCHAR' put: 'CHAR'.
	DataTypeMap at: #'NUMBER' put: 'DECIMAL'.

	"POSTGRES Mappings"
	DataTypeMap at: #'INT4' put: 'INTEGER'.
	DataTypeMap at: #'DOUBLE PRECISION' put: 'FLOAT'.
	DataTypeMap at: #'REAL' put: 'FLOAT'.
	DataTypeMap at: #'CHARACTER VARYING' put: 'VARCHAR'.
	DataTypeMap at: #'CHARACTER' put: 'CHAR'.
	
	"POSTGRES Date and Time mappings"
	DataTypeMap at: #'TIMESTAMP WITHOUT TIME ZONE' put: 'TIMESTAMP'.
	DataTypeMap at: #'TIMESTAMP WITH TIME ZONE' put: 'TIMESTAMP'.
	DataTypeMap at: #'TIMESTAMP WITH LOCAL TIME ZONE' put: 'TIMESTAMP'.

	"MYSQL Date and Time Mappings"
	DataTypeMap at: #'TIME WITHOUT TIME ZONE' put: 'TIME'.
	DataTypeMap at: #'TIME WITH TIME ZONE' put: 'TIME'.
	DataTypeMap at: #'DATETIME' put: 'TIMESTAMP'.
	
! !

!SQueakLConnection class methodsFor: 'table methods' stamp: 'MartinLeo 7/31/2010 12:18'!
makeColumnGeneric: aSQueakLTableColumn
	"This method takes a SQueakLTableColumn, and transform it's data type into a generic SQueakLite data type.
	ie type = genericType within aSQueakLTableColumn.
	
	See SQueakLTableColumn>>type for more information.
	
	It will return UNSUPPORTED if SQueakLite cannot support the appropriate data type."

	aSQueakLTableColumn type: (SQueakLConnection makeGeneric: aSQueakLTableColumn type)! !

!SQueakLConnection class methodsFor: 'table methods' stamp: 'MartinLeo 8/2/2010 10:56'!
makeGeneric: aTypeName
	"This method takes a datatype name, and tries to map it into an existing SQueakLite generic data type.
	It will return UNSUPPORTED if SQueakLite cannot support the appropriate data type.
	
	All data types into MYSQL / Oracle / PostGRES has been extensively mapped, unsupported datatypes are left out ot the dictionary.
	A lookup will cause a not found error which will be caught and treated as an unsupported type.
	
	An extensive list of unsupported data types is included below.
	"

	[^DataTypeMap at: aTypeName asUppercase asSymbol]
	on: Error 
	do: [^'UNSUPPORTED']
	
	"
	Short / Non Standard Dates:
	YEAR
	
	Binary Bit String Storage:
	BINARY
	VARBINARY
	BIT VARYING
	
	Raw / streaming data types:
	CLOB
	NCLOB
	BLOB
	BFILE
	RAW
	LONG RAW
	LONG
	TEXT

	Boolean is not in ANSI SQL standard:
	BOOLEAN	

	Geo-Spatial Types:
	BOX
	PATH
	POINT
	POLYGON
	LSEG
	CIRCLE
	BOX
	LINE

	Date Base Internals:
	ROWID
	UROWID

	Auto Incrementing Sequences (while made popular by MS Access, is not standard)	:
	SERIAL
	BIGSERIAL
	INTERVAL

	Internet Addresses:
	INET
	MACADDR
	CIDR

	Other Miscellaneous Features:		
	ENUM
	MONEY
	BYTEA
	XML Types
	"! !

!SQueakLConnection class methodsFor: 'connection managers' stamp: 'MartinLeo 7/31/2010 23:03'!
newType: aTypeSymbol mapTo: aGenericTypeString
	"This method allows one to register new database type mappings.
	
	For example, to register a MYSQLs SMALLDATETIME type,
	call this:
	
		SQueakLConnection newType: #SMALLDATETIME mapTo: 'DATE'.		

	If a user tries to redefine an existing datatype map to something different, 
	this method will return false, else it will return true.
	"

	DataTypeMap at: aTypeSymbol ifPresent: 
		[((DataTypeMap at: aTypeSymbol) = aGenericTypeString) ifTrue: [^true] ifFalse: [^false]].

	DataTypeMap at: aTypeSymbol put: aGenericTypeString.
	^true.! !

!SQueakLConnection class methodsFor: 'connection managers' stamp: 'MartinLeo 7/10/2010 21:01'!
register: SQueakLConnectionDatabase
	"Registers a new SQueakLConnection Database.
	This object must subclass SQueakLConnection.
	"
	
	|checkForExistingName|
	
	SQueakLConnectionDatabase superclass = SQueakLConnection
		ifFalse: [^self error: 'Wrong connection object passed to SQueakLConnection>>Register.
									 Must be of type SQueakLConnection.'].

	checkForExistingName := self databaseFactory: SQueakLConnectionDatabase connectionDatabase.

	checkForExistingName ifNotNil: [^self error: 'A connection object with the same name already exists!!'].
	
	SQueakLConnectionTypes add: SQueakLConnectionDatabase.! !

!SQueakLConnection methodsFor: 'private' stamp: 'MartinLeo 7/30/2010 23:43'!
cleanColumnDataType: aDataType
	|re|
	
	re := '\((.*)\)' asRegexIgnoringCase.
	^re copy: aDataType replacingMatchesWith: ''
! !

!SQueakLConnection methodsFor: 'private' stamp: 'MartinLeo 8/2/2010 00:06'!
close
	"Closes the connection to the database. See >>open for more details"
	
	accelerated ifFalse: [
		connection close.
		connection disconnect.
		]
! !

!SQueakLConnection methodsFor: 'instance release' stamp: 'MartinLeo 7/28/2010 22:05'!
connect: aHost port: aPort database: aDatabase userName: aUserName userPassword: aUserPassword
	"This method sets up the connection parameters for (open)ing , db operations, (close)ing."

	^DBXConnectionSettings
                host: aHost
                port: aPort
                database: aDatabase
                userName: aUserName 
                userPassword: aUserPassword ! !

!SQueakLConnection methodsFor: 'private' stamp: 'DrewGlass 7/17/2010 17:34'!
connection 	
	^connection! !

!SQueakLConnection methodsFor: 'connection methods' stamp: 'DrewGlass 7/17/2010 17:31'!
connectionName
	^connectionName! !

!SQueakLConnection methodsFor: 'connection methods' stamp: 'DrewGlass 7/17/2010 17:31'!
connectionName: aName
	connectionName := aName! !

!SQueakLConnection methodsFor: 'table methods' stamp: 'MartinLeo 7/28/2010 22:09'!
createView: aSQueakLView
	"Generic ANSI SQL compliant view creation script, sub-classess / connections can override this if necessary."
	
	^self executeDDL: 'CREATE VIEW ', aSQueakLView schemaName , '.', aSQueakLView name, ' AS ',  aSQueakLView viewDef.
	! !

!SQueakLConnection methodsFor: 'table methods' stamp: 'MartinLeo 7/28/2010 22:09'!
dropView: aSQueakLView
	"Generic ANSI SQL compliant view removal command, sub-classess / connections can override this if necessary."
	
	^self executeDDL: 'DROP VIEW ', aSQueakLView schemaName ,'.', aSQueakLView viewName
	! !

!SQueakLConnection methodsFor: 'db methods' stamp: 'DrewGlass 7/13/2010 21:00'!
execute: statement
	"Executes Single DDL statemenmts."	
		
	^connection execute: statement.! !

!SQueakLConnection methodsFor: 'db methods' stamp: 'MartinLeo 7/28/2010 22:12'!
executeDDL: statement
	"
	Executes Single DDL statemenmts.
	
	This is an independent statement execution command. It cannot be inserted in between opens and close.
	
	Returns SQueakLError object, if SQueakLError>>error = False, then the operation succeeded.
	Else look at SQueakLError>>message for details.
	"	
		
	|ddlError|
	ddlError := SQueakLError new.

	self open.
	[connection execute: statement]
		on: Exception 
		do: [:x | ddlError message: (x messageText)].
	self close.

 	^ddlError ! !

!SQueakLConnection methodsFor: 'db methods' stamp: 'MartinLeo 7/28/2010 22:13'!
executeDDLOnly: statement
	"
	Executes Single DDL statemenmts.
	
	This command allows you to run multiple statements in a row.
	It's connection needs to be open before this method can be executed.
	
	Returns SQueakLError object, if SQueakLError>>error = False, then the operation succeeded.
	Else look at SQueakLError>>message for details.
	"	
		
	|ddlError|
	ddlError := SQueakLError new.

	[connection execute: statement]
		on: Exception 
		do: [:x | ddlError message: (x messageText). self close.].

 	^ddlError! !

!SQueakLConnection methodsFor: 'private' stamp: 'MartinLeo 7/28/2010 22:17'!
executeDDLScript: statement
	"This method is not supported. It is used internally by the unit tests to setup test structures in the database.
	
	One can run an entire script without OpenDBX validation by using this method."
	
	self open.
	connection executeDDLScript: statement.
	self close

! !

!SQueakLConnection methodsFor: 'db methods' stamp: 'MartinLeo 7/28/2010 22:25'!
genericQuery: statement result: resultSet
	"Executes any database query.
	This method will detect whether it is a data retreival query or any thing else.	
	If is data retrieval query is executed, the resultSet will be populated, otherwise the SQL statement is just executed as-is"

	|firstWordHeuristics|

	firstWordHeuristics := ((statement subStrings: '$ ') at: 1) asUppercase.
	
	(firstWordHeuristics = 'SELECT' or: [firstWordHeuristics = 'WITH'] or: [firstWordHeuristics = 'SHOW'])
		ifTrue: [^self query: statement result: resultSet]
		ifFalse: [^self executeDDL: statement]! !

!SQueakLConnection methodsFor: 'instance release' stamp: 'MartinLeo 8/1/2010 23:57'!
initialize 
	accelerated := true.! !

!SQueakLConnection methodsFor: 'connection methods' stamp: 'MartinLeo 8/2/2010 00:22'!
isAccelerated
	"determines whether connection pooling is on or off"
	^accelerated! !

!SQueakLConnection methodsFor: 'private' stamp: 'MartinLeo 8/2/2010 11:14'!
open
	"Opens a connection to the database to execute queries, execute DDL statements and read from result sets.
	
	Connection pooling here is primitive, we could build a new connection manager that runs in a 
	seperate process to keep / maintain open connections.
	"

	accelerated ifTrue: [
		[
		connection connect.
		connection open.
		]
		on: SqueakDBXError
		do: [:x |
			(x messageText = 'SqueakDBX: You are already connected. Try disconnecting first')
			ifFalse: [x signal] 
			]
	] 
	ifFalse: [
		connection connect.
		connection open.	
	]! !

!SQueakLConnection methodsFor: 'schema methods' stamp: 'PeterWubbels 7/20/2010 17:20'!
populateSchema: aSQueakLSchema
	aSQueakLSchema tables: (self tables: aSQueakLSchema).
	aSQueakLSchema views: (self views: aSQueakLSchema).
	aSQueakLSchema indexes: (self indexes: aSQueakLSchema).
	aSQueakLSchema detailsPopulated: true.

	^aSQueakLSchema.! !

!SQueakLConnection methodsFor: 'table methods' stamp: 'MartinLeo 7/28/2010 22:31'!
populateTable: aSQueakLTable
	"This is a template method to assist in fully populating a SQueakLTable instance.
	
	Sub classes (which are actual physical connections) will have to specify how to actually populate it, by implementing:
	1. discoverIndex
	2. discoverColumns.
	
	As a heuristic, we do not need to populate all details of a table.
	If a table instance is returned with detailsPopulated = false, this implies that  the details are not populated."
	
	aSQueakLTable indexes: (self discoverIndex: aSQueakLTable).
	aSQueakLTable columns: (self discoverColumns: aSQueakLTable).
	aSQueakLTable detailsPopulated: true.
	^aSQueakLTable ! !

!SQueakLConnection methodsFor: 'private' stamp: 'MartinLeo 7/28/2010 22:32'!
processView: viewDef
		"This is a helper method that simply strips out the CREATE VIEW SOMEVIEW AS from view definitions returned by the database.
		MYSQL returns that (together with /*ALGORITHM=DEF*/), Oracle does not and will not be affected by this helper method."
		
		|defPos|
		
		defPos := viewDef asUppercase findString: 'SELECT'.
		^(viewDef allButFirst: (defPos - 1)) copy
! !

!SQueakLConnection methodsFor: 'db methods' stamp: 'MartinLeo 7/28/2010 22:36'!
query: statement result: resultSet
	"This method is the actual workhorse method behind retrieving data from the database,
	 it will execute the provided SQL statement, and put the data into a SQueakLDynamicResultSet.
	
	Things populated here includes, column names, number of columns, number of rows return and
	of course the data set itself.
	
	Although multiple features are performed here, it is not spilt into multiple methods because
	it is a straight forward data retreival exercise from OpenDBXs resultset."

	|tmpResultSet colNames colCount valueMatrix ddlError|
	
	ddlError := SQueakLError new.
	valueMatrix := OrderedCollection new.
	self open.		
			
	[tmpResultSet := connection execute: statement.	
	
	colCount := tmpResultSet columnCount.
	colNames := tmpResultSet columnDescriptions copy.

	tmpResultSet rowsDo: [ :dbxRow | |tmpRow|
		tmpRow := OrderedCollection new.
		(1 to: colCount) do: [:n | tmpRow add: (dbxRow valueAt: n)].
		valueMatrix add: tmpRow.
		].
	
	resultSet columnCount: colCount.
	resultSet columnNames: colNames.
	resultSet columnValues: valueMatrix.
	
	]
	on: Exception 
	do: [:x | ddlError message: (x messageText)].
	
	[self close]
	on: Exception 
	do: ["nothing"].
	
	^ddlError! !

!SQueakLConnection methodsFor: 'private' stamp: 'MartinLeo 7/29/2010 21:42'!
queryIndexObject: aSQueakLIndex result: resultSet	
		"This method is a private method to queryObject. It converts aSQueakLIndex into a SQueakDynamicResultSet for easy display of data."

		|name indexColumns|
		
		name := OrderedCollection new.
		name add: 'Position'.
		name add: 'Column'.
		
		resultSet columnNames: name.
		resultSet columnCount: 2.
		
		indexColumns := OrderedCollection new.
		
		aSQueakLIndex indexColumns do: [:aSQueakLIndexColumn |
					indexColumns add: (OrderedCollection with: (aSQueakLIndexColumn index) with: (aSQueakLIndexColumn column)).
			].
		
		indexColumns add: (OrderedCollection with: 'Index Type' with: (aSQueakLIndex indexType)).
		resultSet columnValues: indexColumns.
		
		^SQueakLError new.
! !

!SQueakLConnection methodsFor: 'db methods' stamp: 'MartinLeo 7/29/2010 20:57'!
queryObject: aSQueakLObject result: resultSet
	"This is a straight forward dispatcher to the query:result: method.
	It provides a convinent and programmatic means of access to data within database objects.
	No SQL statements needed, just a well formed SQueakLObject (SQueakLTable or SQueakLView)"
	
	|selectStatement|
	
	(aSQueakLObject class = SQueakLIndex) ifTrue: [ ^self queryIndexObject: aSQueakLObject result: resultSet].
	
	selectStatement := 'SELECT * FROM ', aSQueakLObject schemaName ,'.', aSQueakLObject name.
	
	^self query: selectStatement result: resultSet ! !

!SQueakLConnection methodsFor: 'connection methods' stamp: 'MartinLeo 8/2/2010 00:22'!
setAccelerationOff
	"toggles connection pooling / reuse off"
	accelerated := false! !

!SQueakLConnection methodsFor: 'connection methods' stamp: 'MartinLeo 8/2/2010 00:22'!
setAccelerationOn
	"toggles connection pooling / reuse on"
	accelerated := true! !

!SQueakLConnection methodsFor: 'table methods' stamp: 'MartinLeo 7/29/2010 21:54'!
updateTabCols: aSQueakLTable columnAction: aSQueakLTableColumnAction
	"This method modifies a table definition (not structure, it is a helper class that updates SQueakLite meta-data)
	with a provided Value Object named aSQueakLTableColumnAction.
	 Actions are performed as declared by SQueakLTableColumnAction>>action

   	 See SQueakLTableColumnAction for more details.
	"

	|prevTblColDef|

	prevTblColDef := aSQueakLTable columns copy.

	(aSQueakLTableColumnAction action = 'DROP')	
	ifTrue: [
	aSQueakLTable columns: (prevTblColDef reject: [:each | each field = aSQueakLTableColumnAction field]) copy.
	].
	
	(aSQueakLTableColumnAction action = 'ADD')	
	ifTrue: [
	prevTblColDef add: aSQueakLTableColumnAction copy.
	aSQueakLTable columns: prevTblColDef copy.
	].

	(aSQueakLTableColumnAction action = 'MODIFY')	
	ifTrue: [ |i j|
	i := 0.
	prevTblColDef do: [:each |
		i := i + 1. 
		(each field = aSQueakLTableColumnAction field) 
			ifTrue: [j := i].
		].
	prevTblColDef at: j put: (aSQueakLTableColumnAction copy).
	aSQueakLTable columns: prevTblColDef copy.		
	].
	
	(aSQueakLTableColumnAction action = 'RENAME')	
	ifTrue: [ |i j oldColDef|
	i := 0.
	prevTblColDef do: [:each |
		i := i + 1. 
		(each field = aSQueakLTableColumnAction field) 
			ifTrue: [j := i].
		].
	oldColDef := prevTblColDef at: j.
	oldColDef field: (aSQueakLTableColumnAction newName copy).
	
	prevTblColDef at: j put: (oldColDef copy).
	aSQueakLTable columns: prevTblColDef copy.		
	].

! !

!SQueakLConnection methodsFor: 'table methods' stamp: 'MartinLeo 7/29/2010 21:57'!
updateTabCols: aSQueakLTable indexAction: aSQueakLIndexAction

	"This method modifies a tables index definition (not structure, it is a helper class that updates SQueakLite meta-data)
	with a provided Value Object named aSQueakLIndexColumnAction.
	 Actions are performed as declared by SQueakLIndexColumnAction>>action

   	 See SQueakLIndexColumnAction for more details.
	"

	|prevIdxDef|

	prevIdxDef := aSQueakLTable indexes copy.

	(aSQueakLIndexAction action = 'DROP')	
	ifTrue: [
		aSQueakLTable indexes: (prevIdxDef reject: [:each | each indexName = aSQueakLIndexAction indexName]) copy.
	].
	
	(aSQueakLIndexAction action = 'CREATE')	
	ifTrue: [
		prevIdxDef add: aSQueakLIndexAction copy.
		aSQueakLTable indexes: prevIdxDef copy.
	].
! !

Object subclass: #SQueakLDynamicResultSet
	instanceVariableNames: 'columnNames columnValues columnCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLDynamicResultSet commentStamp: 'MartinLeo 7/31/2010 00:31' prior: 0!
A SQueakLDynamicResultSet is xxxxxxxxx.

Instance Variables
	columnCount:		<Object>
	columnNames:		<Object>
	columnValues:		<Object>

columnCount
	- this is the number of *columns* returned

columnNames
	- This is an OrderedCollection of column names.

columnValues
	- This is where the data is kept in, it is basically a matrix, represented by an OrderedCollection of OrderedCollection
	
	
Column Names and Values are stored in the same order. 

If a column name is 
OrderedCollection('A','B')

Then

(columnValues at: aRow#) at: 1 is A's column
and
(columnValues at: aRow#) at: 2 is B's column!

!SQueakLDynamicResultSet class methodsFor: 'instance creation' stamp: 'MartinLeo 7/31/2010 11:43'!
new
	"This Class is the resultset of SQueakLite.	
	Data values are store in a matrix format inside column values.	
	It is stored as an OrderedCollection of OrderedCollections.
	
	initialize will instantiate the matrix (mentioned above)"

	^self basicNew initialize! !

!SQueakLDynamicResultSet methodsFor: 'accessing' stamp: 'MartinLeo 7/14/2010 22:14'!
columnCount
	^ columnCount! !

!SQueakLDynamicResultSet methodsFor: 'accessing' stamp: 'MartinLeo 7/14/2010 22:14'!
columnCount: anObject
	columnCount := anObject! !

!SQueakLDynamicResultSet methodsFor: 'accessing' stamp: 'MartinLeo 7/14/2010 22:15'!
columnNames
	^ columnNames! !

!SQueakLDynamicResultSet methodsFor: 'accessing' stamp: 'MartinLeo 7/14/2010 22:15'!
columnNames: anObject
	columnNames := anObject! !

!SQueakLDynamicResultSet methodsFor: 'accessing' stamp: 'MartinLeo 7/14/2010 22:14'!
columnValues
	^ columnValues! !

!SQueakLDynamicResultSet methodsFor: 'accessing' stamp: 'MartinLeo 7/14/2010 22:14'!
columnValues: anObject
	columnValues := anObject! !

!SQueakLDynamicResultSet methodsFor: 'initialize-release' stamp: 'MartinLeo 7/14/2010 22:16'!
initialize 
	columnCount := 0.
! !

!SQueakLDynamicResultSet methodsFor: 'accessing' stamp: 'MartinLeo 7/15/2010 00:05'!
recordCount
	^columnValues size! !

Object subclass: #SQueakLError
	instanceVariableNames: 'errorMessage errorCondition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLError commentStamp: 'MartinLeo 7/31/2010 00:36' prior: 0!
A SQueakLError is the error class of SQueakLite, it is not meant to work as an Exception type object but as a part of the workflow.

For example, all DDL type methods should return an error object with its error condition appropriately set.

Instance Variables
	errorCondition:		<Object>
	errorMessage:		<Object>

errorCondition
	- On false, life goes on, on true, there is an error condition that needs to be handled.

errorMessage
	- Error message of problem, usually this message is populated by the exception's message that was intercepted 
!

!SQueakLError class methodsFor: 'instance creation' stamp: 'MartinLeo 7/29/2010 22:50'!
new
	"This is SQueakLites error object, errorCondition should be initialized to false"
	^self basicNew initialize ! !

!SQueakLError methodsFor: 'accessing' stamp: 'MartinLeo 7/10/2010 22:23'!
error 
	^errorCondition! !

!SQueakLError methodsFor: 'initialize-release' stamp: 'MartinLeo 7/31/2010 11:48'!
initialize 
	"errorCondition must be initialized to be false for a new error object,
	otherwise connection methods will be confused as to how to cope with errors."
	
	errorCondition := false ! !

!SQueakLError methodsFor: 'accessing' stamp: 'MartinLeo 7/10/2010 22:25'!
message
	^errorMessage ! !

!SQueakLError methodsFor: 'accessing' stamp: 'MartinLeo 7/31/2010 11:49'!
message: aMessage
	"If an error message is set, this means that there is an error condition, so toggle errorCondition to true"
	
	errorCondition := true.
	errorMessage := aMessage! !

Object subclass: #SQueakLIndexColumn
	instanceVariableNames: 'index column'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLIndexColumn commentStamp: 'MartinLeo 7/31/2010 00:39' prior: 0!
A SQueakLIndexColumn is a value object that represents an indexed column in an index.

Instance Variables
	column:		<Object>
	index:		<Object>

column
	- The column that the index affects.

index
	- The position of this column within the index.
!

!SQueakLIndexColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 13:03'!
column 
	^column ! !

!SQueakLIndexColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 13:03'!
column: aColumn
	column := aColumn ! !

!SQueakLIndexColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/31/2010 12:53'!
index
	"Returns the verbatim data type, some times its a numeric, other times its a string,
	depending on how the database stores it. Connection objects will deal with this uncertainity.
	
	This value object is trying to preserve actual values returned by
	the DB for future expandability."

	^index! !

!SQueakLIndexColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/31/2010 11:50'!
index: anIndex
	"Can take either a numeric value or a string, 
	as different databases has different responses.
	connection methods will accomodate."
	
	index := anIndex! !

Object subclass: #SQueakLSavedConnections
	instanceVariableNames: 'connections fileName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLSavedConnections commentStamp: 'DrewGlass 7/28/2010 16:11' prior: 0!
A SQueakLSavedConnections represents a collection of connections saved by the user. Connections are saved to and retrieved from file using a ReferenceStream. Connections can also be deleted.

Instance Variables
	connections:		<OrderedCollection>
	fileName:		<String>
!

!SQueakLSavedConnections class methodsFor: 'information methods' stamp: 'DrewGlass 7/28/2010 09:27'!
defaultFileName
	^'SQueakLiteSavedConnections.obj'! !

!SQueakLSavedConnections class methodsFor: 'instance creation' stamp: 'DrewGlass 7/28/2010 10:39'!
fileName: aString
	^self new fileName: aString.! !

!SQueakLSavedConnections methodsFor: 'accessing' stamp: 'DrewGlass 7/28/2010 10:30'!
connections
	^connections
	! !

!SQueakLSavedConnections methodsFor: 'accessing' stamp: 'DrewGlass 7/28/2010 10:13'!
connectionsAt: anInteger
	^connections at: anInteger.
	! !

!SQueakLSavedConnections methodsFor: 'accessing' stamp: 'DrewGlass 7/28/2010 10:12'!
connectionsDo: aBlock
	^connections do: aBlock.
	! !

!SQueakLSavedConnections methodsFor: 'persistance' stamp: 'DrewGlass 7/31/2010 12:05'!
deleteConnectionAt: anInteger

	connections removeAt: anInteger.
	self saveToFile.! !

!SQueakLSavedConnections methodsFor: 'accessing' stamp: 'MartinLeo 7/31/2010 11:52'!
fileName: aString
	"sets a save file instead of the default"

	| file rr |
	fileName := aString.
	file := FileDirectory default fileNamesMatching: fileName.
   file isEmpty 
		ifTrue: [connections := OrderedCollection new]
		ifFalse: [ 
			rr := ReferenceStream fileNamed: fileName.
			connections := rr next.
			rr close.].! !

!SQueakLSavedConnections methodsFor: 'instance release' stamp: 'DrewGlass 7/28/2010 10:11'!
initialize
	| file rr |
	file := FileDirectory default fileNamesMatching: self class defaultFileName.
   file isEmpty 
		ifTrue: [connections := OrderedCollection new]
		ifFalse: [ 
			rr := ReferenceStream fileNamed: self class defaultFileName.
			connections := rr next.
			rr close.].! !

!SQueakLSavedConnections methodsFor: 'persistance' stamp: 'DrewGlass 7/31/2010 12:05'!
saveConnection: aConnection name: aName

	aConnection connectionName: aName.
	connections addLast: aConnection.
	self saveToFile.! !

!SQueakLSavedConnections methodsFor: 'persistance' stamp: 'DrewGlass 7/31/2010 12:05'!
saveToFile
	| referenceStream |
	
	referenceStream := ReferenceStream fileNamed: fileName.
	[ referenceStream nextPut: connections. ] ensure: [ referenceStream close ].
	referenceStream close.! !

Object subclass: #SQueakLSchema
	instanceVariableNames: 'schemaName connection tables views indexes detailsPopulated'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLSchema commentStamp: 'MartinLeo 7/31/2010 00:43' prior: 0!
A SQueakLSchema is a value object the represents a schema, please read the instance variables description for more information.

Instance Variables
	connection:		<Object>
	detailsPopulated:		<Object>
	indexes:		<Object>
	schemaName:		<Object>
	tables:		<Object>
	views:		<Object>

connection
	- The connection that loaded this schema.

detailsPopulated
	- Whether its just a shell of a schema, or the following details below are populated.

indexes
	- Contains a collection of indexes in this schema

schemaName
	- The name of this schema

tables
	- Contains a collection of tables in this schema

views
	- Contains a collection of views in this schema
!

!SQueakLSchema class methodsFor: 'instance creation' stamp: 'PeterWubbels 7/17/2010 15:02'!
named: aName withConnection: aConnection
	^ SQueakLSchema new schema: aName; connection: aConnection.! !

!SQueakLSchema methodsFor: 'accessing' stamp: 'PeterWubbels 7/17/2010 15:34'!
connection
	^connection.! !

!SQueakLSchema methodsFor: 'accessing' stamp: 'PeterWubbels 7/17/2010 14:58'!
connection: aConnection
	connection := aConnection.! !

!SQueakLSchema methodsFor: 'accessing' stamp: 'PeterWubbels 7/17/2010 15:34'!
detailsPopulated
	^detailsPopulated.! !

!SQueakLSchema methodsFor: 'accessing' stamp: 'PeterWubbels 7/17/2010 15:32'!
detailsPopulated: schemaDetailsPopulated
	detailsPopulated := schemaDetailsPopulated.! !

!SQueakLSchema methodsFor: 'accessing' stamp: 'PeterWubbels 7/21/2010 01:48'!
indexes
	detailsPopulated 
		ifFalse: [connection populateSchema: self].

	^indexes.! !

!SQueakLSchema methodsFor: 'accessing' stamp: 'PeterWubbels 7/20/2010 17:20'!
indexes: schemaIndexes
	indexes := schemaIndexes.	! !

!SQueakLSchema methodsFor: 'initialize-release' stamp: 'PeterWubbels 7/17/2010 15:35'!
initialize
	detailsPopulated := false.! !

!SQueakLSchema methodsFor: 'accessing' stamp: 'MartinLeo 7/10/2010 21:37'!
schema
	^schemaName ! !

!SQueakLSchema methodsFor: 'accessing' stamp: 'MartinLeo 7/10/2010 21:38'!
schema: aSchemaName
	^schemaName := aSchemaName ! !

!SQueakLSchema methodsFor: 'accessing' stamp: 'PeterWubbels 7/17/2010 15:36'!
tables
	detailsPopulated 
		ifFalse: [connection populateSchema: self].

	^tables.! !

!SQueakLSchema methodsFor: 'accessing' stamp: 'PeterWubbels 7/17/2010 15:31'!
tables: schemaTables
	tables := schemaTables.! !

!SQueakLSchema methodsFor: 'accessing' stamp: 'PeterWubbels 7/20/2010 16:31'!
views
	detailsPopulated 
		ifFalse: [connection populateSchema: self].

	^views.! !

!SQueakLSchema methodsFor: 'accessing' stamp: 'PeterWubbels 7/20/2010 16:31'!
views: schemaViews
	views := schemaViews.
	! !

Object subclass: #SQueakLSchemaObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLSchemaObject commentStamp: 'MartinLeo 7/31/2010 01:04' prior: 0!
A SQueakLSchemaObject is a helper object that assists in the setting of selected objects for a connection in the GUI.

Instance Variables
!

SQueakLSchemaObject subclass: #SQueakLIndex
	instanceVariableNames: 'indexName tableName indexType indexDetails schemaName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLIndex commentStamp: 'MartinLeo 7/31/2010 00:50' prior: 0!
A SQueakLIndex is a value object that represents a single database index. For more details, please examine the description of its instance variables below.

Instance Variables
	indexDetails:		<Object>
	indexName:		<Object>
	indexType:		<Object>
	schemaName:		<Object>
	tableName:		<Object>

indexDetails
	- An OrderedCollection of SQueakLIndexColumns is stored here.
		(which is the details of the index itself)

indexName
	- The name of the index

indexType
	- The type of the index. (Indexes can be implemented differently eg BTREE BITMAP R-TREE)

schemaName
	- The name of the schema this index is residing in

tableName
	- The name of the table that this index is indexing on.
!

!SQueakLIndex class methodsFor: 'instance creation' stamp: 'MartinLeo 7/11/2010 13:20'!
new
 	^self basicNew initialize ! !

!SQueakLIndex methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 13:08'!
addDetail: aSQueakLIndexColumn
	indexDetails add: aSQueakLIndexColumn! !

!SQueakLIndex methodsFor: 'accessing' stamp: 'MartinLeo 7/31/2010 11:53'!
indexColumns
	"Returns an OrderedCollection of SQueakLIndexColumns"
	
	^indexDetails.! !

!SQueakLIndex methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 12:48'!
indexName
	^indexName! !

!SQueakLIndex methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 12:48'!
indexName: anIndexName
	indexName := anIndexName! !

!SQueakLIndex methodsFor: 'accessing' stamp: 'MartinLeo 7/13/2010 23:37'!
indexType
	^indexType ! !

!SQueakLIndex methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 13:11'!
indexType: anIndexType
	indexType := anIndexType ! !

!SQueakLIndex methodsFor: 'initialize-release' stamp: 'MartinLeo 7/11/2010 13:08'!
initialize 
	indexDetails := OrderedCollection new.! !

!SQueakLIndex methodsFor: 'accessing' stamp: 'PeterWubbels 7/20/2010 17:14'!
name
	^ self indexName.! !

!SQueakLIndex methodsFor: 'accessing' stamp: 'MartinLeo 7/18/2010 10:50'!
schemaName
	^ schemaName! !

!SQueakLIndex methodsFor: 'accessing' stamp: 'MartinLeo 7/18/2010 10:50'!
schemaName: anObject
	schemaName := anObject! !

!SQueakLIndex methodsFor: 'accessing' stamp: 'MartinLeo 7/28/2010 21:12'!
tableName
	^tableName! !

!SQueakLIndex methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 12:48'!
tableName: aTableName
	tableName := aTableName ! !

SQueakLIndex subclass: #SQueakLIndexAction
	instanceVariableNames: 'action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLIndexAction commentStamp: 'MartinLeo 7/31/2010 01:05' prior: 0!
A SQueakLIndexAction is a decorator type of a value object that wraps an action around an index.
This class will not perform anything.
It is passed to a method that will accept this object and then perform the relevant operation.

Instance Variables
	action:		<Object>

action
	- Only CREATE or DROP is supported. Not all databases all you to modify an index, which in essence is basically a DROP followed by a CREATE.

!

!SQueakLIndexAction methodsFor: 'accessing' stamp: 'MartinLeo 7/18/2010 11:32'!
action
	^ action! !

!SQueakLIndexAction methodsFor: 'accessing' stamp: 'MartinLeo 7/29/2010 22:57'!
action: anObject
	"An Action object (be it for Index or Column) is simply a index or column value object with an added action field.
	
	The reason is this, operation methods can act on this action object, and without any conversion just add it into
	an established collection of indexes / columns (as it is a subclass).
	
	Subsquently, we can look back and see a history of changes on a particular table.
	"
	action := anObject! !

!SQueakLSchemaObject methodsFor: 'accessing' stamp: 'PeterWubbels 7/20/2010 17:26'!
name
	self subclassResponsibility.! !

SQueakLSchemaObject subclass: #SQueakLTable
	instanceVariableNames: 'tableName schema indexes columns detailsPopulated primaryKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLTable commentStamp: 'MartinLeo 7/31/2010 00:58' prior: 0!
A SQueakLTable is a value object the reflects a database table. 
Please see the table descriptions for more details.

Instance Variables
	columns:		<Object>
	detailsPopulated:		<Object>
	indexes:		<Object>
	primaryKey:		<Object>
	schema:		<Object>
	tableName:		<Object>

columns
	- An OrderedCollection of SQueakLTableColumn (represents the column details of the table)

detailsPopulated
	- can be either true (all columnds and details fully populated, it is set by populateTable) or false ()

indexes
	- An OrderedCollectrion of SQueakLIndex (represents the indexes, which can be multiple, defined on a table)

primaryKey
	- Unused for now

schema
	- Stores the schema object that contains this table

tableName
	- Stores the name of this table.
!

!SQueakLTable class methodsFor: 'instance creation' stamp: 'MartinLeo 7/11/2010 11:34'!
new
	^self basicNew initialize! !

!SQueakLTable methodsFor: 'accessing' stamp: 'MartinLeo 7/29/2010 22:59'!
columns
	"Returns an OrderedCollection of SQueakLTableColumn."
	
	^columns! !

!SQueakLTable methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 13:44'!
columns: tableColumns
	columns := tableColumns! !

!SQueakLTable methodsFor: 'accessing' stamp: 'MartinLeo 7/29/2010 22:58'!
detailsPopulated
	"if true, columns and indexes are populated, otherwise only the table name and schema is populated."
	^detailsPopulated ! !

!SQueakLTable methodsFor: 'accessing' stamp: 'MartinLeo 7/13/2010 21:37'!
detailsPopulated: detailsPopulatedOrNot
	^detailsPopulated := detailsPopulatedOrNot ! !

!SQueakLTable methodsFor: 'accessing' stamp: 'MartinLeo 7/29/2010 22:59'!
indexes
	"Returns an ordered collection of SQueakLIndex"
	^indexes! !

!SQueakLTable methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 13:44'!
indexes: tableIndexes
	indexes := tableIndexes ! !

!SQueakLTable methodsFor: 'initialize-release' stamp: 'MartinLeo 7/18/2010 12:28'!
initialize
	detailsPopulated := false.
	indexes := OrderedCollection new.
	columns := OrderedCollection new! !

!SQueakLTable methodsFor: 'accessing' stamp: 'PeterWubbels 7/20/2010 17:05'!
name
	^self table.! !

!SQueakLTable methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 12:18'!
schema
	^schema! !

!SQueakLTable methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 12:17'!
schema: aSchema
	schema := aSchema! !

!SQueakLTable methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 13:17'!
schemaName
	^schema schema! !

!SQueakLTable methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 11:11'!
table
	^tableName ! !

!SQueakLTable methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 11:11'!
table: aTableName
	tableName := aTableName! !

SQueakLSchemaObject subclass: #SQueakLView
	instanceVariableNames: 'viewName viewDef schema'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLView commentStamp: 'MartinLeo 7/31/2010 01:02' prior: 0!
A SQueakLView is a value object that represents a database view.
Please see the instance variable descriptions for more information.

Instance Variables
	schema:		<Object>
	viewDef:		<Object>
	viewName:		<Object>

schema
	- Stores a reference to the schema that contains this view

viewDef
	- Stores the view definition.
	*** CAUTION, some complicated views are extremely long and are stored as long text / clob type formats in a database.
			as opendbx does not support that, sometimes view definitions might be truncated. (eg Oracle, truncated after 4000 characters)

viewName
	- The name of the view.

Views cannot be modified, it is provided as a definition and "compiled" by a database.
We can only CREATE / DROP a view.!

!SQueakLView methodsFor: 'accessing' stamp: 'PeterWubbels 7/20/2010 17:05'!
name
	^self viewName.! !

!SQueakLView methodsFor: 'accessing' stamp: 'PeterWubbels 7/20/2010 16:48'!
schema 
	^schema.! !

!SQueakLView methodsFor: 'accessing' stamp: 'PeterWubbels 7/20/2010 16:48'!
schema: aSchema
	schema := aSchema.! !

!SQueakLView methodsFor: 'accessing' stamp: 'PeterWubbels 7/20/2010 16:59'!
schemaName
	^ schema schema.! !

!SQueakLView methodsFor: 'accessing' stamp: 'MartinLeo 7/29/2010 23:00'!
viewDef
	"Only the Select portion is returned. (CREATE VIEW ... AS is dropped)"
	
	^viewDef! !

!SQueakLView methodsFor: 'accessing' stamp: 'MartinLeo 7/19/2010 21:38'!
viewDef: anObject
	viewDef := anObject! !

!SQueakLView methodsFor: 'accessing' stamp: 'MartinLeo 7/19/2010 21:37'!
viewName
	^ viewName! !

!SQueakLView methodsFor: 'accessing' stamp: 'MartinLeo 7/19/2010 21:37'!
viewName: anObject
	viewName := anObject! !

Object subclass: #SQueakLTableColumn
	instanceVariableNames: 'field type charLength precision scale genericType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLTableColumn commentStamp: 'MartinLeo 7/31/2010 01:14' prior: 0!
A SQueakLTableColumn is a value object that reflects the structure of a column within a table.

Data types have different fields to track its structure, for SQueakLite generic data types,

An integer stores no further information.

A decimal will store a precision and a scale.

A varchar will store a character length.

A char will store a character length.

A date will store no arguments.

A timestamp will store no arguments.

A float will store a precision value.

Instance Variables
	charLength:		<Object>
	field:		<Object>
	genericType:		<Object>
	precision:		<Object>
	scale:		<Object>
	type:		<Object>

charLength
	- For string based types, stores the length or maximum length (for variable strings).

field
	- The name of this column / field

genericType
	- Stores an equivalent SQueakLite data type to the type instance variable.

precision
	- Precision is stored here, some db's return a number as string others, just a number. There is protection against this in the methods.

scale
	- Scale is stored here, some db's return a number as string others, just a number. There is protection against this in the methods.

type
	- Stores the actual DB type used to represent an object in a database.
!

!SQueakLTableColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 14:09'!
charLength
	^charLength! !

!SQueakLTableColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 14:11'!
charLength: aCharLength
	charLength := aCharLength ! !

!SQueakLTableColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 13:35'!
field
	^field! !

!SQueakLTableColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 13:35'!
field: columnName
	field := columnName ! !

!SQueakLTableColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/30/2010 23:07'!
genericType
	^ genericType! !

!SQueakLTableColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/30/2010 23:07'!
genericType: anObject
	genericType := anObject! !

!SQueakLTableColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 14:10'!
precision
	^precision! !

!SQueakLTableColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 14:11'!
precision: aPrecision
	precision:= aPrecision! !

!SQueakLTableColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 14:10'!
scale
	^scale! !

!SQueakLTableColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 14:10'!
scale: aScale
	scale := aScale! !

!SQueakLTableColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/11/2010 13:34'!
type
	^type! !

!SQueakLTableColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/31/2010 12:15'!
type: aDBType
	"As per the class description, a column contains two type definitions, the actual database definition and SQueakLites generic definition.
	We populate genericType automatically when type (actual database defn) is populated."

	type := aDBType.
	genericType := (SQueakLConnection makeGeneric: aDBType)
	
! !

!SQueakLTableColumn methodsFor: 'accessing' stamp: 'MartinLeo 7/30/2010 23:19'!
typeAndDetails
	"returns type details like: decimal(14,2)"
	^SQueakLConnection formatColumnDefn: self! !

SQueakLTableColumn subclass: #SQueakLTableColumnAction
	instanceVariableNames: 'action newName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SQueakLite-Base'!

!SQueakLTableColumnAction commentStamp: 'MartinLeo 7/31/2010 01:08' prior: 0!
A SQueakLTableColumnAction is a decorator type of a value object that wraps an action around a Table Column.
This class will not perform anything.
It is passed to a method that will accept this object and then perform the relevant operation.


Instance Variables
	action:		<Object>
	newName:		<Object>

action
	- ADD, DROP, RENAME and MODFIY a column is supported.
	Note: Modification is not safe, some types cannot be casted to other types.

newName
	- For rename operations, we need to be able to inform the method that will be performing the renaming operation what the new name is.
		This variable will store that new name.
!

!SQueakLTableColumnAction class methodsFor: 'instance creation' stamp: 'PeterWubbels 7/29/2010 23:41'!
withColumn: aSQueakLTableColumn
	"Initializes and returns a new SQueakLiteTableColumnAction with the data in aSQueakLTableColumn."
	
	^ SQueakLTableColumnAction new
		charLength: aSQueakLTableColumn charLength;
		field: aSQueakLTableColumn field;
		precision: aSQueakLTableColumn precision;
		scale: aSQueakLTableColumn scale;
		type: aSQueakLTableColumn type.! !

!SQueakLTableColumnAction class methodsFor: 'instance creation' stamp: 'PeterWubbels 7/30/2010 11:49'!
withColumn: aSQueakLTableColumn andAction: anAction
	"Initializes and returns a new SQueakLiteTableColumnAction with the data in aSQueakLTableColumn and the action name (string) in anAction."
	
	^ SQueakLTableColumnAction new
		charLength: aSQueakLTableColumn charLength;
		field: aSQueakLTableColumn field;
		precision: aSQueakLTableColumn precision;
		scale: aSQueakLTableColumn scale;
		type: aSQueakLTableColumn type;
		action: anAction.! !

!SQueakLTableColumnAction methodsFor: 'accessing' stamp: 'MartinLeo 7/29/2010 22:57'!
action
	"An Action object (be it for Index or Column) is simply a index or column value object with an added action field.
	
	The reason is this, operation methods can act on this action object, and without any conversion just add it into
	an established collection of indexes / columns (as it is a subclass).
	
	Subsquently, we can look back and see a history of changes on a particular table.
	"
	^ action! !

!SQueakLTableColumnAction methodsFor: 'accessing' stamp: 'MartinLeo 7/17/2010 13:05'!
action: anObject
	action := anObject! !

!SQueakLTableColumnAction methodsFor: 'accessing' stamp: 'MartinLeo 7/29/2010 22:57'!
newName
	"This is a temporary field to store a new name during a renaming operation."
	^ newName! !

!SQueakLTableColumnAction methodsFor: 'accessing' stamp: 'MartinLeo 7/17/2010 14:41'!
newName: anObject
	newName := anObject! !
SQueakLConnection initialize!
